@@ -1,7 +1,7 @@
 /*
  *	command.c
  *
- *	Command Execute Module
+ *	command executing module
  */
 
 #include "fd.h"
@@ -13,6 +13,10 @@
 #include "system.h"
 #endif
 
+#ifndef	_NOPTY
+#include "termemu.h"
+#endif
+
 extern int curcolumns;
 extern int mark;
 extern off_t marksize;
@@ -31,12 +35,19 @@ extern int win_y;
 extern char *destpath;
 extern char *histfile;
 extern int savehist;
+#ifndef	_NOTRADLAYOUT
 extern int tradlayout;
+#endif
 extern int sizeinfo;
+extern int wheader;
 extern char fullpath[];
+extern int hideclock;
 #ifndef	_NOORIGSHELL
 extern int fdmode;
 #endif
+#ifndef	_NOPTY
+extern int ptymode;
+#endif
 
 static VOID NEAR replacefname __P_((char *));
 static int cur_up __P_((char *));
@@ -71,6 +82,7 @@ static int dotfile_mode __P_((char *));
 static int fileflg_mode __P_((char *));
 static int log_dir __P_((char *));
 static int log_top __P_((char *));
+static VOID NEAR clearscreen __P_((VOID_A));
 #ifndef	PAGER
 static VOID NEAR dump __P_((char *));
 #endif
@@ -87,6 +99,9 @@ static int help_message __P_((char *));
 #ifndef	_NOCUSTOMIZE
 static int edit_config __P_((char *));
 #endif
+#ifndef	_NOPTY
+static int confirmpty __P_((VOID_A));
+#endif
 static int quit_system __P_((char *));
 static int make_dir __P_((char *));
 static int copy_file __P_((char *));
@@ -128,6 +143,11 @@ static VOID NEAR movewin __P_((int));
 static int split_window __P_((char *));
 static int next_window __P_((char *));
 #endif
+#ifndef	_NOEXTRAWIN
+static int widen_window __P_((char *));
+static int narrow_window __P_((char *));
+static int kill_window __P_((char *));
+#endif
 static int warning_bell __P_((char *));
 static int no_operation __P_((char *));
 
@@ -265,6 +285,11 @@ bindtable bindlist[MAXBINDTABLE] = {
 #ifndef	_NOSPLITWIN
 	{'/',		SPLIT_WINDOW,	255},
 	{'^',		NEXT_WINDOW,	255},
+#endif
+#ifndef	_NOEXTRAWIN
+	{'W',		WIDEN_WINDOW,	255},
+	{'N',		NARROW_WINDOW,	255},
+	{'K',		KILL_WINDOW,	255},
 #endif
 	{K_HOME,	MARK_ALL,	255},
 	{K_END,		MARK_REVERSE,	255},
@@ -309,6 +334,7 @@ char *arg;
 	else
 #endif
 	if (filepos < 0) filepos = 0;
+
 	return((filepos != old) ? 2 : 0);
 }
 
@@ -335,6 +361,7 @@ char *arg;
 	else
 #endif
 	if (filepos >= maxfile) filepos = maxfile - 1;
+
 	return((filepos != old) ? 2 : 0);
 }
 
@@ -371,6 +398,7 @@ char *arg;
 				filepos = maxfile - 1;
 		}
 	}
+
 	return((filepos != old) ? 2 : 0);
 }
 
@@ -400,6 +428,7 @@ char *arg;
 #endif
 	if (filepos >= 0) /*EMPTY*/;
 	else filepos = old % r;
+
 	return((filepos != old) ? 2 : 0);
 }
 
@@ -416,6 +445,7 @@ char *arg;
 		filepos = old;
 		return(warning_bell(arg));
 	}
+
 	return(2);
 }
 
@@ -432,6 +462,7 @@ char *arg;
 		filepos = old;
 		return(warning_bell(arg));
 	}
+
 	return(2);
 }
 
@@ -441,6 +472,7 @@ char *arg;
 {
 	if (filepos == 0) return(0);
 	filepos = 0;
+
 	return(2);
 }
 
@@ -450,6 +482,7 @@ char *arg;
 {
 	if (filepos == maxfile - 1) return(0);
 	filepos = maxfile - 1;
+
 	return(2);
 }
 
@@ -459,6 +492,7 @@ char *arg;
 {
 	if (fnameofs <= 0) return(0);
 	fnameofs--;
+
 	return(2);
 }
 
@@ -480,7 +514,8 @@ char *arg;
 			if (!(filelist[filepos].linkname)) i = 0;
 			else {
 				i = strlen(filelist[filepos].linkname);
-				if (i > sizeof(tmp) - 1) i = sizeof(tmp) - 1;
+				if (i > (int)sizeof(tmp) - 1)
+					i = (int)sizeof(tmp) - 1;
 				strncpy(tmp, filelist[filepos].linkname, i);
 			}
 		}
@@ -498,11 +533,13 @@ char *arg;
 			m = filelist[filepos].st_mode;
 			if ((m & S_IFMT) == S_IFDIR || (m & S_IFMT) == S_IFLNK
 			|| (m & S_IFMT) == S_IFSOCK || (m & S_IFMT) == S_IFIFO
-			|| (m & (S_IXUSR | S_IXGRP | S_IXOTH))) i--;
+			|| (m & (S_IXUSR | S_IXGRP | S_IXOTH)))
+				i--;
 		}
 	}
 	if (i >= strlen3(filelist[filepos].name) - fnameofs) return(0);
 	fnameofs++;
+
 	return(2);
 }
 
@@ -511,6 +548,7 @@ static int one_column(arg)
 char *arg;
 {
 	curcolumns = 1;
+
 	return(2);
 }
 
@@ -519,6 +557,7 @@ static int two_columns(arg)
 char *arg;
 {
 	curcolumns = 2;
+
 	return(2);
 }
 
@@ -527,6 +566,7 @@ static int three_columns(arg)
 char *arg;
 {
 	curcolumns = 3;
+
 	return(2);
 }
 
@@ -535,6 +575,7 @@ static int five_columns(arg)
 char *arg;
 {
 	curcolumns = 5;
+
 	return(2);
 }
 
@@ -542,19 +583,19 @@ static VOID NEAR markcount(VOID_A)
 {
 #ifndef	_NOTRADLAYOUT
 	if (istradlayout()) {
-		locate(TC_MARK, TL_PATH);
-		cprintf2("%<*d", TD_MARK, mark);
-		locate(TC_MARK + TD_MARK + TW_MARK, TL_PATH);
-		cprintf2("%<'*qd", TD_SIZE, marksize);
+		Xlocate(TC_MARK, TL_PATH);
+		Xcprintf2("%<*d", TD_MARK, mark);
+		Xlocate(TC_MARK + TD_MARK + TW_MARK, TL_PATH);
+		Xcprintf2("%<'*qd", TD_SIZE, marksize);
 
 		return;
 	}
 #endif
-	locate(C_MARK + W_MARK, L_STATUS);
-	cprintf2("%<*d", D_MARK, mark);
+	Xlocate(C_MARK + W_MARK, L_STATUS);
+	Xcprintf2("%<*d", D_MARK, mark);
 	if (sizeinfo) {
-		locate(C_SIZE + W_SIZE, L_SIZE);
-		cprintf2("%<'*qd", D_SIZE, marksize);
+		Xlocate(C_SIZE + W_SIZE, L_SIZE);
+		Xcprintf2("%<'*qd", D_SIZE, marksize);
 	}
 }
 
@@ -575,6 +616,7 @@ char *arg;
 			marksize -= getblock(filelist[filepos].st_size);
 	}
 	markcount();
+
 	return(2);
 }
 
@@ -583,6 +625,7 @@ char *arg;
 {
 	mark_file(arg);
 	if (filepos < maxfile - 1) filepos++;
+
 	return(2);
 }
 
@@ -591,8 +634,10 @@ char *arg;
 {
 	mark_file(arg);
 	if (filepos < maxfile - 1
-	&& filepos / FILEPERPAGE == (filepos + 1) / FILEPERPAGE) filepos++;
+	&& filepos / FILEPERPAGE == (filepos + 1) / FILEPERPAGE)
+		filepos++;
 	else filepos = (filepos / FILEPERPAGE) * FILEPERPAGE;
+
 	return(2);
 }
 
@@ -617,6 +662,7 @@ char *arg;
 		}
 	}
 	markcount();
+
 	return(2);
 }
 
@@ -635,6 +681,7 @@ char *arg;
 				marksize += getblock(filelist[i].st_size);
 		}
 	markcount();
+
 	return(2);
 }
 
@@ -654,6 +701,7 @@ char *mes, *arg;
 
 	re = regexp_init(wild, -1);
 	free(wild);
+
 	return(re);
 }
 
@@ -674,6 +722,7 @@ char *arg;
 		}
 	regexp_free(re);
 	markcount();
+
 	return(3);
 }
 
@@ -685,7 +734,9 @@ char *arg;
 	else
 #endif
 	if (!isdir(&(filelist[filepos]))
-	|| !strcmp(filelist[filepos].name, ".")) return(warning_bell(arg));
+	|| !strcmp(filelist[filepos].name, "."))
+		return(warning_bell(arg));
+
 	return(5);
 }
 
@@ -698,6 +749,7 @@ char *arg;
 	else
 #endif
 	replacefname(NULL);
+
 	return(5);
 }
 
@@ -716,6 +768,7 @@ char *arg;
 		memcpy((char *)&(filelist[i]),
 			(char *)&(filelist[i + 1]), sizeof(namelist));
 	if (filepos >= maxfile) filepos = maxfile - 1;
+
 	return(2);
 }
 
@@ -733,6 +786,7 @@ char *arg;
 	memcpy((char *)&(filelist[filepos + 1]),
 		(char *)&(filestack[--stackdepth]), sizeof(namelist));
 	chgorder = 1;
+
 	return(2);
 }
 
@@ -741,6 +795,7 @@ static int symlink_mode(arg)
 char *arg;
 {
 	dispmode ^= F_SYMLINK;
+
 	return(4);
 }
 
@@ -749,6 +804,7 @@ static int filetype_mode(arg)
 char *arg;
 {
 	dispmode ^= F_FILETYPE;
+
 	return(2);
 }
 
@@ -757,6 +813,7 @@ static int dotfile_mode(arg)
 char *arg;
 {
 	dispmode ^= F_DOTFILE;
+
 	return(4);
 }
 
@@ -765,6 +822,7 @@ static int fileflg_mode(arg)
 char *arg;
 {
 	dispmode ^= F_FILEFLAG;
+
 	return(2);
 }
 
@@ -818,6 +876,7 @@ char *arg;
 	}
 #endif
 	replacefname(NULL);
+
 	return(4);
 }
 
@@ -830,23 +889,52 @@ char *arg;
 	path = strdup2(_SS_);
 	if (chdir3(path, 1) < 0) error(path);
 	replacefname(path);
+
 	return(4);
 }
 
+static VOID NEAR clearscreen(VOID_A)
+{
+#ifndef	_NOPTY
+	int i, yy;
+
+	if (ptymode) {
+		yy = filetop(win);
+		for (i = 0; i < FILEPERROW; i++) {
+			Xlocate(0, yy + i);
+			Xputterm(L_CLEAR);
+		}
+	}
+	else
+#endif	/* !_NOPTY */
+	Xputterms(T_CLEAR);
+	Xtflush();
+}
+
 #ifndef	PAGER
 static VOID NEAR dump(file)
 char *file;
 {
 	FILE *fp;
 	char *buf, *prompt;
-	int i, len;
+	int i, len, min, max;
 
 	if (!(fp = Xfopen(file, "r"))) {
 		warning(-1, file);
 		return;
 	}
-	putterms(t_clear);
-	tflush();
+	clearscreen();
+
+	min = 0;
+	max = n_line - 1;
+# ifndef	_NOPTY
+	if (ptymode) {
+		min = filetop(win);
+		max = min + FILEPERROW;
+	}
+# endif
+
+	Xtflush();
 	buf = NEXT_K;
 	i = strlen3(file);
 	len = strlen2(buf);
@@ -855,45 +943,49 @@ char *file;
 	strncpy3(prompt, file, &i, 0);
 	strcat(prompt, buf);
 
-	i = 0;
+	i = min;
 	while ((buf = fgets2(fp, 0))) {
-		locate(0, i);
-		putterm(l_clear);
-		cprintf2("%.*k", n_column, buf);
+		Xlocate(0, i);
+		Xputterm(L_CLEAR);
+		Xcprintf2("%.*k", n_column, buf);
 		free(buf);
-		if (++i >= n_line - 1) {
-			i = 0;
+		if (++i >= max) {
+			i = min;
+			hideclock = 1;
 			if (!yesno(prompt)) break;
 		}
 	}
 
 	if (Xfeof(fp)) {
-		while (i < n_line + 1) {
-			locate(0, i++);
-			putterm(l_clear);
+		while (i < max) {
+			Xlocate(0, i++);
+			Xputterm(L_CLEAR);
 		}
 	}
 	Xfclose(fp);
 	free(prompt);
 }
-#endif
+#endif	/* !PAGER */
 
 static int NEAR execenv(env, arg)
 char *env, *arg;
 {
 	char *command;
 
-	if (!(command = getenv2(env))) return(0);
-	putterms(t_clear);
-	tflush();
-	execmacro(command, arg, 1, 0, 1);
+	if (!(command = getenv2(env)) || !*command) return(0);
+	clearscreen();
+	ptymacro(command, arg, F_NOCONFIRM | F_IGNORELIST);
+
 	return(1);
 }
 
 static int NEAR execshell(VOID_A)
 {
+#ifndef	_NOPTY
+	int x, y, min, max;
+#endif
 	char *sh;
-	int mode, nl, ret;
+	int mode, ret, wastty;
 
 	sh = getenv2("FD_SHELL");
 #if	MSDOS
@@ -917,18 +1009,29 @@ static int NEAR execshell(VOID_A)
 #endif
 
 	sigvecset(0);
-	mode = termmode(0);
-	nl = stdiomode();
-	kanjifputs(SHEXT_K, stderr);
-	fputnl(stderr);
+	mode = Xtermmode(0);
+	wastty = isttyiomode;
+	kanjiputs(SHEXT_K);
+#ifndef	_NOPTY
+	if (ptymode) {
+		min = filetop(win);
+		max = min + FILEPERROW;
+		y = max;
+		if (!isttyiomode) Xttyiomode(0);
+		if (getxy(&x, &y) < 0 || --y >= max) y = max - 1;
+		regionscroll(C_SCROLLFORW, 1, 0, y, min, max - 1);
+	}
+	else
+#endif	/* !_NOPTY */
+	Xcputnl();
+	Xtflush();
+	if (isttyiomode) Xstdiomode();
+	ret = ptysystem(sh);
 #ifdef	_NOORIGSHELL
-	ret = system(sh);
-#else
-	if (sh) ret = dosystem(sh);
-	else ret = shell_loop(1);
+	checkscreen(-1, -1);
 #endif
-	ttyiomode(nl);
-	termmode(mode);
+	if (wastty) Xttyiomode(wastty - 1);
+	Xtermmode(mode);
 	sigvecset(1);
 
 	return(ret);
@@ -963,10 +1066,12 @@ char *arg;
 
 	if (!execenv("FD_PAGER", filelist[filepos].name)) {
 #ifdef	PAGER
-		execmacro(PAGER, filelist[filepos].name, 1, 0, 1);
+		ptymacro(PAGER, filelist[filepos].name,
+			F_NOCONFIRM | F_IGNORELIST);
 #else
 		do {
 			dump(filelist[filepos].name);
+			hideclock = 1;
 		} while (!yesno(PEND_K));
 #endif
 	}
@@ -980,6 +1085,7 @@ char *arg;
 	else
 #endif
 	/*EMPTY*/;
+
 	return(2);
 }
 
@@ -1002,7 +1108,8 @@ char *arg;
 #endif
 	if (!execenv("FD_EDITOR", filelist[filepos].name)) {
 #ifdef	EDITOR
-		execmacro(EDITOR, filelist[filepos].name, 1, 0, 1);
+		ptymacro(EDITOR, filelist[filepos].name,
+			F_NOCONFIRM | F_IGNORELIST);
 #endif
 	}
 #ifndef	_NODOSDRIVE
@@ -1012,6 +1119,7 @@ char *arg;
 		removetmp(dir, filelist[filepos].name);
 	}
 #endif
+
 	return(4);
 }
 
@@ -1078,6 +1186,7 @@ char *arg;
 		free(dupl);
 		chgorder = 1;
 	}
+
 	return(2);
 }
 
@@ -1096,9 +1205,10 @@ char *arg;
 	if (underhome(NULL) <= 0 && !yesno(HOMOK_K)) return(1);
 	arrangedir(fs);
 	chgorder = 0;
+
 	return(4);
 }
-#endif
+#endif	/* !_NOWRITEFS */
 
 /*ARGSUSED*/
 static int reread_dir(arg)
@@ -1112,6 +1222,7 @@ char *arg;
 #ifndef	_NODOSDRIVE
 	if ((drive = dospath3(""))) flushdrv(drive, NULL);
 #endif
+
 	return(4);
 }
 
@@ -1125,6 +1236,7 @@ char *arg;
 		return(1);
 	}
 	customize();
+
 	return(4);
 }
 #endif
@@ -1138,9 +1250,22 @@ char *arg;
 #else
 	help(archivefile != NULL);
 #endif
+
 	return(2);
 }
 
+#ifndef	_NOPTY
+static int confirmpty(VOID_A)
+{
+	int i;
+
+	for (i = 0; i < MAXWINDOWS; i++) if (ptylist[i].pid) break;
+	if (i < MAXWINDOWS && !yesno(KILL_K)) return(1);
+
+	return(0);
+}
+#endif	/* !_NOPTY */
+
 /*ARGSUSED*/
 static int quit_system(arg)
 char *arg;
@@ -1157,9 +1282,9 @@ char *arg;
 		char *str[3];
 		int val[3];
 
-		locate(0, L_MESLINE);
-		putterm(l_clear);
-		kanjiputs(QUIT_K);
+		Xlocate(0, L_MESLINE);
+		Xputterm(L_CLEAR);
+		Xkanjiputs(QUIT_K);
 		val[0] = 0;
 		val[1] = 1;
 		val[2] = 2;
@@ -1173,7 +1298,12 @@ char *arg;
 #endif	/* _NOORIGSHELL */
 	if (!yesno(QUIT_K)) return(1);
 
+#ifndef	_NOPTY
+	if (confirmpty()) return(1);
+#endif
+
 	if (savehist > 0) savehistory(0, histfile);
+
 	return((n) ? -2 : -1);
 }
 
@@ -1191,6 +1321,7 @@ char *arg;
 
 	if (mkdir2(path, 0777) < 0) warning(-1, path);
 	free(path);
+
 	return(4);
 }
 
@@ -1263,6 +1394,7 @@ char *arg;
 		file = strdup2("..");
 	}
 	replacefname(file);
+
 	return(4);
 }
 
@@ -1280,12 +1412,13 @@ char *arg;
 	else if (isdir(&(filelist[filepos]))) return(warning_bell(arg));
 	else {
 		cp = DELFL_K;
-		len = strlen2(cp) - (sizeof("%.*s") - 1);
+		len = strlen2(cp) - ((int)sizeof("%.*s") - 1);
 		if (!yesno(cp, n_lastcolumn - len, filelist[filepos].name))
 			return(1);
 		filepos = applyfile(rmvfile, NULL);
 	}
 	if (filepos >= maxfile && (filepos -= 2) < 0) filepos = 0;
+
 	return(4);
 }
 
@@ -1298,7 +1431,7 @@ char *arg;
 	if (!isdir(&(filelist[filepos])) || isdotdir(filelist[filepos].name))
 		return(warning_bell(arg));
 	cp = DELDR_K;
-	len = strlen2(cp) - (sizeof("%.*s") - 1);
+	len = strlen2(cp) - ((int)sizeof("%.*s") - 1);
 	if (!yesno(cp, n_lastcolumn - len, filelist[filepos].name)) return(1);
 	removepolicy = 0;
 #ifndef	NOSYMLINK
@@ -1317,6 +1450,7 @@ char *arg;
 	if (!applydir(filelist[filepos].name, rmvfile, NULL, rmvdir, 3, NULL))
 		filepos++;
 	if (filepos >= maxfile && (filepos -= 2) < 0) filepos = 0;
+
 	return(4);
 }
 
@@ -1328,11 +1462,11 @@ char *arg;
 	if (arg && *arg) wild = strdup2(arg);
 	else if (!(wild = inputstr(FINDF_K, 0, 0, "*", -1))) return(1);
 
-	if (
 #ifndef	_NOARCHIVE
-	*wild != '/' &&
+	if (*wild == '/') /*EMPTY*/;
+	else
 #endif
-	strdelim(wild, 1)) {
+	if (strdelim(wild, 1)) {
 		warning(ENOENT, wild);
 		free(wild);
 		return(1);
@@ -1343,6 +1477,7 @@ char *arg;
 		free(wild);
 		findpattern = NULL;
 	}
+
 	return(4);
 }
 
@@ -1366,6 +1501,7 @@ char *arg;
 
 	replacefname(strdup2(cp));
 	free(destpath);
+
 	return(4);
 }
 
@@ -1377,21 +1513,22 @@ char *arg;
 
 	if (arg) com = strdup2(arg);
 	else {
-		ttyiomode(1);
+		Xttyiomode(1);
 		com = inputshellloop(-1, NULL);
-		ttyiomode(0);
+		Xttyiomode(0);
 		if (!com) return(1);
 	}
 	if (*com) {
-		ret = execusercomm(com, filelist[filepos].name, 0, -1, 0);
+		ret = ptyusercomm(com, filelist[filepos].name, F_ARGSET);
 		ret = (ret < 0) ? 1 :
-			(internal_status >= -1) ? internal_status: 4;
+			((internal_status < -1) ? 4 : internal_status);
 	}
 	else {
 		execshell();
 		ret = 4;
 	}
 	free(com);
+
 	return(ret);
 }
 
@@ -1424,9 +1561,9 @@ char *arg;
 #ifdef	CWDINPATH
 	len = (!isdir(&(filelist[filepos])) && isexec(&(filelist[filepos])))
 		? strlen(filelist[filepos].name) + 1 : 0;
-	ttyiomode(1);
+	Xttyiomode(1);
 	com = inputshellloop(len, filelist[filepos].name);
-	ttyiomode(0);
+	Xttyiomode(0);
 #else
 	if (isdir(&(filelist[filepos])) || !isexec(&(filelist[filepos]))) {
 		len = 0;
@@ -1439,9 +1576,9 @@ char *arg;
 		tmp[1] = _SC_;
 		strcpy(&(tmp[2]), filelist[filepos].name);
 	}
-	ttyiomode(1);
+	Xttyiomode(1);
 	com = inputshellloop(len, tmp);
-	ttyiomode(0);
+	Xttyiomode(0);
 	if (tmp != filelist[filepos].name) free(tmp);
 #endif
 	if (!com) return(1);
@@ -1455,8 +1592,8 @@ char *arg;
 	if (archivefile) {
 		if (!(dir = tmpunpack(1))) ret = -1;
 		else {
-			ret = execusercomm(com,
-				filelist[filepos].name, 0, -1, 1);
+			ret = ptyusercomm(com, filelist[filepos].name,
+				F_ARGSET | F_IGNORELIST);
 			removetmp(dir, NULL);
 		}
 	}
@@ -1465,16 +1602,18 @@ char *arg;
 #ifndef	_NODOSDRIVE
 	if ((drive = tmpdosdupl("", &dir, 1)) < 0) ret = -1;
 	else if (drive) {
-		ret = execusercomm(com, filelist[filepos].name, 0, -1, 1);
+		ret = ptyusercomm(com, filelist[filepos].name,
+			F_ARGSET | F_IGNORELIST);
 		removetmp(dir, filelist[filepos].name);
 	}
 	else
 #endif
 	{
-		ret = execusercomm(com, filelist[filepos].name, 0, -1, 0);
+		ret = ptyusercomm(com, filelist[filepos].name, F_ARGSET);
 	}
-	ret = (ret < 0) ? 1 : (internal_status >= -1) ? internal_status: 4;
+	ret = (ret < 0) ? 1 : ((internal_status < -1) ? 4 : internal_status);
 	free(com);
+
 	return(ret);
 }
 
@@ -1483,6 +1622,7 @@ static int launch_file(arg)
 char *arg;
 {
 	if (!launcher()) return(view_file(arg));
+
 	return(4);
 }
 
@@ -1502,10 +1642,11 @@ char *arg;
 	ret = pack(file);
 	free(file);
 	if (ret < 0) {
-		putterm(t_bell);
+		Xputterm(T_BELL);
 		return(1);
 	}
 	if (!ret) return(1);
+
 	return(4);
 }
 
@@ -1517,11 +1658,14 @@ char *arg;
 #endif
 	int ret;
 
-	if (archivefile) ret = unpack(archivefile, NULL, arg, 0, 0);
+	if (archivefile) ret = unpack(archivefile, NULL, arg,
+		0, F_ARGSET | F_ISARCH | F_NOADDOPT);
 	else if (isdir(&(filelist[filepos]))) return(warning_bell(arg));
-	else ret = unpack(fnodospath(path, filepos), NULL, arg, 0, 1);
+	else ret = unpack(fnodospath(path, filepos), NULL, arg,
+		0, F_ARGSET | F_ISARCH | F_NOADDOPT | F_IGNORELIST);
 	if (ret < 0) return(warning_bell(arg));
 	if (!ret) return(1);
+
 	return(4);
 }
 
@@ -1534,13 +1678,16 @@ char *arg;
 #endif
 	int ret;
 
-	if (archivefile) ret = unpack(archivefile, NULL, NULL, 1, 0);
+	if (archivefile) ret = unpack(archivefile, NULL, NULL,
+		1, F_ARGSET | F_ISARCH | F_NOADDOPT);
 	else if (isdir(&(filelist[filepos]))) return(warning_bell(arg));
-	else ret = unpack(fnodospath(path, filepos), NULL, NULL, 1, 1);
+	else ret = unpack(fnodospath(path, filepos), NULL, NULL,
+		1, F_ARGSET | F_ISARCH | F_NOADDOPT | F_IGNORELIST);
 	if (ret <= 0) {
 		if (ret < 0) warning_bell(arg);
 		return(3);
 	}
+
 	return(4);
 }
 #endif	/* !_NOTREE */
@@ -1562,6 +1709,7 @@ char *arg;
 	ret = infofs(path);
 	free(path);
 	if (!ret) return(1);
+
 	return(3);
 }
 
@@ -1582,9 +1730,9 @@ char *arg;
 			if (ismark(&(filelist[i]))) break;
 		if (i >= maxfile) i = filepos;
 		flag = 1;
-		locate(0, L_INFO);
-		putterm(l_clear);
-		kanjiputs(ATTRM_K);
+		Xlocate(0, L_INFO);
+		Xputterm(L_CLEAR);
+		Xkanjiputs(ATTRM_K);
 		if (selectstr(&flag, 2, 35, str, val) != K_CR) return(1);
 	}
 	else {
@@ -1603,8 +1751,8 @@ char *arg;
 		if (FILEPERROW < WFILEMINATTR) return(4);
 		return(2);
 	}
-
 	applyfile(setattr, NULL);
+
 	return(4);
 }
 
@@ -1625,6 +1773,7 @@ char *arg;
 	if (findpattern) free(findpattern);
 	findpattern = NULL;
 	replacefname(NULL);
+
 	return(4);
 }
 #endif	/* !_NOTREE */
@@ -1679,6 +1828,7 @@ char *arg;
 
 	free(dev);
 	if (ret <= 0) return(1);
+
 	return(4);
 }
 #endif	/* !_NOARCHIVE */
@@ -1688,12 +1838,13 @@ static int search_forw(arg)
 char *arg;
 {
 	isearch = 1;
-	locate(0, L_HELP);
-	putterm(l_clear);
-	putterm(t_standout);
-	win_x = kanjiputs(SEAF_K);
+	Xlocate(0, L_HELP);
+	Xputterm(L_CLEAR);
+	Xputterm(T_STANDOUT);
+	win_x = Xkanjiputs(SEAF_K);
 	win_y = L_HELP;
-	putterm(end_standout);
+	Xputterm(END_STANDOUT);
+
 	return(0);
 }
 
@@ -1702,12 +1853,13 @@ static int search_back(arg)
 char *arg;
 {
 	isearch = -1;
-	locate(0, L_HELP);
-	putterm(l_clear);
-	putterm(t_standout);
-	win_x = kanjiputs(SEAB_K);
+	Xlocate(0, L_HELP);
+	Xputterm(L_CLEAR);
+	Xputterm(T_STANDOUT);
+	win_x = Xkanjiputs(SEAB_K);
 	win_y = L_HELP;
-	putterm(end_standout);
+	Xputterm(END_STANDOUT);
+
 	return(0);
 }
 
@@ -1738,41 +1890,92 @@ int oldwin;
 	if (winvar[oldwin].v_fullpath) free(winvar[oldwin].v_fullpath);
 	winvar[oldwin].v_fullpath = strdup2(fullpath);
 	if (winvar[win].v_fullpath) strcpy(fullpath, winvar[win].v_fullpath);
-#ifndef	_NOARCHIVE
+# ifndef	_NOARCHIVE
 	if (winvar[oldwin].v_archivedir) free(winvar[oldwin].v_archivedir);
 	winvar[oldwin].v_archivedir = strdup2(archivedir);
 	if (winvar[win].v_archivedir)
 		strcpy(archivedir, winvar[win].v_archivedir);
-#endif
+# endif
+}
+
+int nextwin(VOID_A)
+{
+	int oldwin;
+
+	if (windows <= 1) return(-1);
+	oldwin = win;
+	if (++win >= windows) win = 0;
+	duplwin(oldwin);
+	movewin(oldwin);
+	if (chdir3(fullpath, 1) < 0) lostcwd(fullpath);
+
+	return(0);
 }
 
 /*ARGSUSED*/
 static int split_window(arg)
 char *arg;
 {
+	winvartable tmp;
 	int i, oldwin;
 
 	oldwin = win;
-	if (windows < MAXWINDOWS) {
-		win = windows++;
-		if (FILEPERROW < WFILEMIN) {
-			windows--;
-			win = oldwin;
-			warning(0, NOROW_K);
-			return(1);
-		}
-		duplwin(oldwin);
-		movewin(oldwin);
-	}
-	else {
+	if (windows >= MAXWINDOWS) {
+# ifdef	_NOEXTRAWIN
+#  ifndef	_NOPTY
+		if (confirmpty()) return(1);
+#  endif
 		if (win > 0) {
 			shutwin(0);
 			win = 0;
 			duplwin(oldwin);
 		}
 		for (i = 1; i < windows; i++) shutwin(i);
+#  ifndef	_NOPTY
+		killallpty();
+#  endif
 		windows = 1;
+		calcwin();
+# else	/* !_NOEXTRAWIN */
+		warning(0, NOWIN_K);
+		return(1);
+# endif	/* !_NOEXTRAWIN */
 	}
+# ifdef	_NOEXTRAWIN
+	else if (fileperrow(windows + 1) < WFILEMIN)
+# else
+	else if (winvar[win].v_fileperrow < WFILEMIN * 2 + 1)
+# endif
+	{
+		warning(0, NOROW_K);
+		return(1);
+	}
+	else {
+		win++;
+		memcpy((char *)&tmp, (char *)&(winvar[windows]),
+			sizeof(winvartable));
+		memmove((char *)&(winvar[win + 1]), (char *)&(winvar[win]),
+			(windows++ - win) * sizeof(winvartable));
+		memcpy((char *)&(winvar[win]), (char *)&tmp,
+			sizeof(winvartable));
+
+# ifdef	_NOEXTRAWIN
+		calcwin();
+# else
+		i = (winvar[oldwin].v_fileperrow - 1) / 2;
+		winvar[win].v_fileperrow =
+			(winvar[oldwin].v_fileperrow - 1) - i;
+		winvar[oldwin].v_fileperrow = i;
+# endif
+
+# ifndef	_NOPTY
+		insertwin(win, windows);
+		changewsize(wheader, windows);
+# endif
+		duplwin(oldwin);
+		movewin(oldwin);
+	}
+
 	return(4);
 }
 
@@ -1780,23 +1983,106 @@ char *arg;
 static int next_window(arg)
 char *arg;
 {
-	int oldwin;
+	if (nextwin() < 0) return(0);
+# ifndef	_NOPTY
+	if (ptylist[win].pid && ptylist[win].status < 0) {
+		rewritefile(0);
+		VOID_C frontend();
+		if (internal_status >= -1) return(internal_status);
+	}
+# endif
 
-	if (windows <= 1) return(0);
-	oldwin = win;
-	if (++win >= windows) win = 0;
-	duplwin(oldwin);
-	movewin(oldwin);
-	if (chdir3(fullpath, 1) < 0) lostcwd(fullpath);
 	return(4);
 }
 #endif	/* !_NOSPLITWIN */
 
+#ifndef	_NOEXTRAWIN
+static int widen_window(arg)
+char *arg;
+{
+	int n, next;
+
+	if (windows <= 1) return(0);
+	if (!arg || (n = atoi2(arg)) <= 0) n = 1;
+	if ((next = win + 1) >= windows) next = 0;
+	if (winvar[next].v_fileperrow - n < WFILEMIN)
+		n = winvar[next].v_fileperrow - WFILEMIN;
+	if (n <= 0) return(warning_bell(arg));
+
+	winvar[next].v_fileperrow -= n;
+	winvar[win].v_fileperrow += n;
+# ifndef	_NOPTY
+	changewsize(wheader, windows);
+# endif
+
+	return(4);
+}
+
+static int narrow_window(arg)
+char *arg;
+{
+	int n, next;
+
+	if (windows <= 1) return(0);
+	if (!arg || (n = atoi2(arg)) <= 0) n = 1;
+	if ((next = win + 1) >= windows) next = 0;
+	if (FILEPERROW - n < WFILEMIN) n = FILEPERROW - WFILEMIN;
+	if (n <= 0) return(warning_bell(arg));
+
+	winvar[next].v_fileperrow += n;
+	winvar[win].v_fileperrow -= n;
+# ifndef	_NOPTY
+	changewsize(wheader, windows);
+# endif
+
+	return(4);
+}
+
+/*ARGSUSED*/
+static int kill_window(arg)
+char *arg;
+{
+	winvartable tmp;
+	int prev;
+
+	if (windows <= 1) return(0);
+	memcpy((char *)&tmp, (char *)&(winvar[win]), sizeof(winvartable));
+	memmove((char *)&(winvar[win]), (char *)&(winvar[win + 1]),
+		(--windows - win) * sizeof(winvartable));
+	memcpy((char *)&(winvar[windows]), (char *)&tmp, sizeof(winvartable));
+
+	if ((prev = win - 1) < 0) prev = windows - 1;
+	winvar[prev].v_fileperrow += winvar[windows].v_fileperrow + 1;
+
+# ifndef	_NOPTY
+	deletewin(win, windows);
+	changewsize(wheader, windows);
+# endif
+
+	win = prev;
+	duplwin(windows);
+	movewin(windows);
+	shutwin(windows);
+	if (chdir3(fullpath, 1) < 0) lostcwd(fullpath);
+
+# ifndef	_NOPTY
+	if (ptylist[win].pid && ptylist[win].status < 0) {
+		rewritefile(0);
+		VOID_C frontend();
+		if (internal_status >= -1) return(internal_status);
+	}
+# endif
+
+	return(4);
+}
+#endif	/* !_NOEXTRAWIN */
+
 /*ARGSUSED*/
 static int warning_bell(arg)
 char *arg;
 {
-	putterm(t_bell);
+	Xputterm(T_BELL);
+
 	return(0);
 }
 
