@@ -1,7 +1,7 @@
 /*
  *	fd.h
  *
- *	Configuration File for FD
+ *	configuration file for FD
  */
 
 #include <stdio.h>
@@ -9,18 +9,11 @@
 #include <errno.h>
 #include "machine.h"
 
-#ifndef	__SYS_TYPES_STAT_H_
-#define	__SYS_TYPES_STAT_H_
-#include <sys/types.h>
-#include <sys/stat.h>
+#ifndef	FD
+#define	FD		2
 #endif
-
-#ifndef	NOUNISTDH
-#include <unistd.h>
-#endif
-
-#ifndef	NOSTDLIBH
-#include <stdlib.h>
+#ifndef	DEFRC
+#define	DEFRC		"/etc/fd2rc"
 #endif
 
 #if	FD < 2
@@ -35,12 +28,17 @@
 #define	_NOBROWSE
 #define	_NOEXTRAMACRO
 #define	_NOTRADLAYOUT
+#define	_NOEXTRAWIN
+#define	_NOPTY
 #endif	/* FD < 2 */
 
+#ifndef	__FD_PRIMAL__
+#include "types.h"
 #include "printf.h"
+#include "kctype.h"
 #include "pathname.h"
 #include "term.h"
-#include "types.h"
+#endif
 
 #ifdef	DEBUG
 extern VOID mtrace __P_ ((VOID));
@@ -52,15 +50,15 @@ extern char *_mtrace_file;
 #if	MSDOS
 # ifdef	BSPATHDELIM
 #  if	FD >= 2
-#  define	RUNCOMFILE	"~\\fd2.rc"
+#  define	FD_RCFILE	"~\\fd2.rc"
 #  else
-#  define	RUNCOMFILE	"~\\fd.rc"
+#  define	FD_RCFILE	"~\\fd.rc"
 #  endif
 # else	/* !BSPATHDELIM */
 #  if	FD >= 2
-#  define	RUNCOMFILE	"~/fd2.rc"
+#  define	FD_RCFILE	"~/fd2.rc"
 #  else
-#  define	RUNCOMFILE	"~/fd.rc"
+#  define	FD_RCFILE	"~/fd.rc"
 #  endif
 # endif	/* !BSPATHDELIM */
 #define	TMPPREFIX	"FD"
@@ -68,9 +66,9 @@ extern char *_mtrace_file;
 #define	DOSTMPPREFIX	'D'
 #else	/* !MSDOS */
 # if	FD >= 2
-# define	RUNCOMFILE	"~/.fd2rc"
+# define	FD_RCFILE	"~/.fd2rc"
 # else
-# define	RUNCOMFILE	"~/.fdrc"
+# define	FD_RCFILE	"~/.fdrc"
 # endif
 #define	TMPPREFIX	"fd"
 #define	ARCHTMPPREFIX	"ar"
@@ -110,7 +108,6 @@ extern char *_mtrace_file;
 #define	INHERITCOPY	0
 #define	ADJTTY		0
 #define	USEGETCURSOR	0
-#define	WINDOWS		1
 #define	DEFCOLUMNS	2
 #define	MINFILENAME	12
 #if	MSDOS
@@ -126,8 +123,8 @@ extern char *_mtrace_file;
 #define	DIRHIST		50
 #define	SAVEHIST	50
 #define	DIRCOUNTLIMIT	50
-#define	SECOND		0
 #define	DOSDRIVE	0
+#define	SECOND		0
 #define	TRADLAYOUT	0
 #define	SIZEINFO	0
 #define	ANSICOLOR	0
@@ -139,8 +136,16 @@ extern char *_mtrace_file;
 #else
 #define	TMPDIR		"/tmp"
 #endif
+#define	TMPUMASK	022
 #define	RRPATH		""
 #define	PRECEDEPATH	""
+#if	FD >= 2
+#define	PROMPT		"$ "
+#else
+#define	PROMPT		"sh#"
+#endif
+#define	PROMPT2		"> "
+#define	DUMBSHELL	0
 #define	UNICODEBUFFER	0
 #define	SJISPATH	""
 #define	EUCPATH		""
@@ -155,14 +160,9 @@ extern char *_mtrace_file;
 #define	UTF8PATH	""
 #define	UTF8MACPATH	""
 #define	NOCONVPATH	""
-#if	FD >= 2
-#define	PROMPT		"$ "
-#else
-#define	PROMPT		"sh#"
-#endif
-#define	PROMPT2		"> "
-#define	TMPUMASK	022
-#define	DUMBSHELL	0
+#define	PTYMODE		0
+#define	PTYTERM		"vt100"
+#define	PTYMENUKEY	-1
 
 
 /****************************************************************
@@ -180,7 +180,13 @@ extern char *_mtrace_file;
 #endif
 #define	MAXSELECTSTRS	16
 #define	MAXSTACK	5
+#ifdef	_NOEXTRAWIN
 #define	MAXWINDOWS	2
+#else
+#define	MAXWINDOWS	5
+#endif
+#define	MAXHISTNO	MAXTYPE(short)
+#define	MAXINVOKEARGS	MAXWINDOWS
 
 #ifdef	_NOSPLITWIN
 #undef	MAXWINDOWS
@@ -191,20 +197,29 @@ extern char *_mtrace_file;
 # endif
 #endif
 
+#ifdef	_NOSPLITWIN
+#define	_NOEXTRAWIN
+#endif
+
 
 /****************************************************************
  *	Screen layout parameter					*
  ****************************************************************/
 #define	FILEPERLINE	(curcolumns)
-#ifndef	_NOSPLITWIN
-#define	FILEPERROW	((n_line - WHEADER - WFOOTER + 1) / windows - 1)
+#ifdef	_NOSPLITWIN
+#define	fileperrow(w)	(n_line - wheader - WFOOTER)
 #else
-#define	FILEPERROW	(n_line - WHEADER - WFOOTER)
+#define	fileperrow(w)	((n_line - wheader - WFOOTER + 1) / (w) - 1)
 #endif
 #define	FILEPERPAGE	(FILEPERLINE * FILEPERROW)
 
+#ifdef	_NOTRADLAYOUT
+#define	istradlayout()	(0)
+#define	hassizeinfo()	(sizeinfo)
+#else
 #define	istradlayout()	(tradlayout && n_column >= WCOLUMNSTD)
 #define	hassizeinfo()	(sizeinfo || istradlayout())
+#endif
 #define	WHEADERMIN	3
 #define	WHEADERMAX	4
 #define	WHEADER		(WHEADERMIN + ((hassizeinfo()) ? 1 : 0))
@@ -225,12 +240,6 @@ extern char *_mtrace_file;
 #define	WFILEMINCUSTOM	4
 #define	WFILEMINATTR	(WMODELINE + 5)
 #define	WFILEMIN	1
-#ifndef	_NOSPLITWIN
-#define	LFILETOP	(WHEADER + (win * (FILEPERROW + 1)))
-#else
-#define	LFILETOP	(WHEADER)
-#endif
-#define	LFILEBOTTOM	(LFILETOP + FILEPERROW)
 #define	L_STACK		(n_line - 3)
 #define	L_HELP		(n_line - 2)
 #define	L_INFO		(n_line - 1)
@@ -396,3 +405,7 @@ extern char *_mtrace_file;
 #if	MSDOS
 #define	_USEDOSCOPY
 #endif
+
+#if	MSDOS || defined (NOSELECT)
+#define	_NOPTY
+#endif
