@@ -1,11 +1,12 @@
 /*
  *	file.c
  *
- *	File Access Module
+ *	file accessing module
  */
 
 #include <fcntl.h>
 #include "fd.h"
+#include "termio.h"
 #include "func.h"
 #include "kanji.h"
 
@@ -75,6 +76,7 @@ static int NEAR cpfile __P_((char *, char *, struct stat *, struct stat *));
 static VOID changemes __P_((VOID_A));
 static int NEAR genrand __P_((int));
 static int dounlink __P_((char *));
+static int dormdir __P_((char *));
 #ifndef	_NOWRITEFS
 static int NEAR isexist __P_((char *));
 static int NEAR realdirsiz __P_((char *, int, int, int, int));
@@ -107,6 +109,7 @@ char *path, *file;
 	path[0] = '.';
 	path[1] = _SC_;
 	strcpy(&(path[2]), file);
+
 	return(path);
 }
 #endif	/* _USEDOSEMU */
@@ -132,6 +135,7 @@ gid_t gid;
 	else if (isgroupmember(gid)) mode >>= 3;
 #endif	/* !NOUID */
 	if (dir && !(mode & F_ISEXE)) mode &= ~(F_ISRED | F_ISWRI);
+
 	return(mode & 007);
 }
 
@@ -154,7 +158,8 @@ namelist *namep;
 		memcpy((char *)&lst, (char *)&st, sizeof(struct stat));
 #if	!MSDOS
 	if ((lst.st_mode & S_IFMT) == S_IFCHR
-	|| (lst.st_mode & S_IFMT) == S_IFBLK) namep -> flags |= F_ISDEV;
+	|| (lst.st_mode & S_IFMT) == S_IFBLK)
+		namep -> flags |= F_ISDEV;
 #endif
 
 	namep -> st_mode = lst.st_mode;
@@ -178,8 +183,8 @@ namelist *namep;
 #else
 		logical_access((u_int)(st.st_mode), st.st_uid, st.st_gid);
 #endif
-
 	namep -> tmpflags |= F_STAT;
+
 	return(0);
 }
 
@@ -206,9 +211,11 @@ CONST VOID_P vp2;
 	if (namep1 -> name[0] == '.' && namep1 -> name[1] == '\0') return(-1);
 	if (namep2 -> name[0] == '.' && namep2 -> name[1] == '\0') return(1);
 	if (namep1 -> name[0] == '.' && namep1 -> name[1] == '.'
-	&& namep1 -> name[2] == '\0') return(-1);
+	&& namep1 -> name[2] == '\0')
+		return(-1);
 	if (namep2 -> name[0] == '.' && namep2 -> name[1] == '.'
-	&& namep2 -> name[2] == '\0') return(1);
+	&& namep2 -> name[2] == '\0')
+		return(1);
 
 	switch (sorton & 7) {
 		case 5:
@@ -300,8 +307,8 @@ CONST VOID_P vp2;
 			tmp = 0;
 			break;
 	}
-
 	if (sorton > 7) tmp = -tmp;
+
 	return(tmp);
 }
 #endif
@@ -335,6 +342,7 @@ char *arcregstr;
 	}
 #endif
 	else dp = Xreaddir(dirp);
+
 	return(dp);
 }
 
@@ -354,11 +362,10 @@ char *buf;
 	}
 	if (!homedir) {
 		if (!(cp = gethomedir())) return(-1);
-		if (
 #ifndef	_NODOSDRIVE
-		dospath2(cp) ||
+		if (dospath2(cp)) return(-1);
 #endif
-		_chdir2(cp) < 0) return(-1);
+		if (_chdir2(cp) < 0) return(-1);
 		homedir = getwd2();
 		if (_chdir2(cwd) < 0) lostcwd(cwd);
 	}
@@ -377,6 +384,7 @@ char *buf;
 	free(homedir);
 	homedir = NULL;
 #endif
+
 	return((cp) ? 1 : 0);
 }
 
@@ -413,8 +421,8 @@ char *dir;
 			return(-1);
 		}
 	}
-
 	entryhist(1, realpath2(dir, tmp, 0), 1);
+
 	return(0);
 }
 
@@ -496,6 +504,7 @@ struct stat *stp;
 	}
 #endif	/* !NOUID */
 	if (ret < 0) errno = duperrno;
+
 	return(ret);
 }
 
@@ -509,6 +518,7 @@ char *path;
 	dosdrive = 0;
 	n = Xchdir(path);
 	dosdrive = dupdosdrive;
+
 	return(n);
 }
 
@@ -522,6 +532,7 @@ char *path;
 	dosdrive = 0;
 	cp = Xgetwd(path);
 	dosdrive = dupdosdrive;
+
 	return(cp);
 }
 
@@ -535,6 +546,7 @@ struct stat *stp;
 	dosdrive = 0;
 	n = Xlstat(path, stp);
 	dosdrive = dupdosdrive;
+
 	return(n);
 }
 
@@ -548,6 +560,7 @@ int mode;
 	dosdrive = 0;
 	n = Xmkdir(path, mode);
 	dosdrive = dupdosdrive;
+
 	return(n);
 }
 
@@ -560,6 +573,7 @@ char *path;
 	dosdrive = 0;
 	n = Xrmdir(path);
 	dosdrive = dupdosdrive;
+
 	return(n);
 }
 #endif	/* !_NODOSDRIVE */
@@ -595,6 +609,7 @@ char *src, *dest;
 	struct stat st1, st2;
 
 	if (Xstat(src, &st1) < 0 || Xstat(dest, &st2) < 0) return(0);
+
 	return(st1.st_dev == st2.st_dev && st1.st_ino == st2.st_ino);
 }
 #endif	/* !NODIRLOOP */
@@ -616,29 +631,11 @@ char *src, *dest;
 	}
 
 	path[len] = '\0';
+
 	return(Xsymlink(path, dest));
 }
 #endif	/* !NOSYMLINK */
 
-int safewrite(fd, buf, size)
-int fd;
-char *buf;
-int size;
-{
-	int n;
-
-	n = Xwrite(fd, buf, size);
-#if	MSDOS
-	if (n >= 0 && n < size) {
-		n = -1;
-		errno = ENOSPC;
-	}
-#else
-	if (n < 0 && errno == EINTR) n = size;
-#endif
-	return(n);
-}
-
 /*ARGSUSED*/
 static int NEAR cpfile(src, dest, stp1, stp2)
 char *src, *dest;
@@ -681,16 +678,21 @@ struct stat *stp1, *stp2;
 		return(-1);
 	}
 
+#ifdef	FAKEUNINIT
+	duperrno = errno;	/* fake for -Wuninitialized */
+#endif
 	for (;;) {
-		while ((i = Xread(fd1, buf, BUFSIZ)) < 0 && errno == EINTR);
-		duperrno = errno;
-		if (i < BUFSIZ) break;
-		if ((i = safewrite(fd2, buf, BUFSIZ)) < 0) {
+		if ((i = sureread(fd1, buf, BUFSIZ)) <= 0) {
 			duperrno = errno;
 			break;
 		}
+		if (surewrite(fd2, buf, i) < 0) {
+			i = -1;
+			duperrno = errno;
+			break;
+		}
+		if (i < BUFSIZ) break;
 	}
-	if (i > 0 && (i = safewrite(fd2, buf, i)) < 0) duperrno = errno;
 
 	Xclose(fd2);
 	Xclose(fd1);
@@ -710,6 +712,7 @@ struct stat *stp1, *stp2;
 	if (inheritcopy && touchfile(dest, stp1) < 0) return(-1);
 # endif
 #endif
+
 	return(0);
 }
 
@@ -759,6 +762,7 @@ struct stat *stp1, *stp2;
 #ifdef	HAVEFLAGS
 	stp1 -> st_flags = (u_long)-1;
 #endif
+
 	return (touchfile(dest, stp1));
 }
 
@@ -800,7 +804,7 @@ int len;
 	int i, j, c;
 
 	if (!buf) {
-		for (i = 0; i < sizeof(seq) / sizeof(char); i++) {
+		for (i = 0; i < (int)sizeof(seq) / sizeof(char); i++) {
 			j = genrand(sizeof(seq) / sizeof(char));
 			c = seq[i];
 			seq[i] = seq[j];
@@ -814,6 +818,7 @@ int len;
 		}
 		buf[i] = '\0';
 	}
+
 	return(buf);
 }
 
@@ -852,9 +857,9 @@ char *dir;
 		}
 	}
 	if (!tmpfilename) {
-		n = sizeof(TMPPREFIX) - 1;
+		n = (int)sizeof(TMPPREFIX) - 1;
 		strncpy(cp, TMPPREFIX, n);
-		len = sizeof(path) - 1 - (cp - path);
+		len = (int)sizeof(path) - 1 - (cp - path);
 		if (len > MAXTMPNAMLEN) len = MAXTMPNAMLEN;
 		len -= n;
 		genrandname(NULL, 0);
@@ -873,7 +878,7 @@ char *dir;
 	}
 
 	strncpy((cp = strcatdelim(path)), dir, n);
-	len = sizeof(path) - 1 - (cp - path);
+	len = (int)sizeof(path) - 1 - (cp - path);
 	if (len > MAXTMPNAMLEN) len = MAXTMPNAMLEN;
 	len -= n;
 	genrandname(NULL, 0);
@@ -893,6 +898,7 @@ char *dir;
 		nodosrmdir(path);
 		errno = n;
 	}
+
 	return(-1);
 }
 
@@ -913,6 +919,7 @@ char *dir;
 	}
 	else if (errno != ENOTEMPTY && errno != EEXIST && errno != EACCES)
 		return(-1);
+
 	return(0);
 }
 
@@ -925,7 +932,7 @@ char *file;
 	path[0] = '\0';
 	if (mktmpdir(path) < 0) return(-1);
 	cp = strcatdelim(path);
-	len = sizeof(path) - 1 - (cp - path);
+	len = (int)sizeof(path) - 1 - (cp - path);
 	if (len > MAXTMPNAMLEN) len = MAXTMPNAMLEN;
 	genrandname(NULL, 0);
 
@@ -942,6 +949,7 @@ char *file;
 	duperrno = errno;
 	rmtmpdir(NULL);
 	errno = duperrno;
+
 	return(-1);
 }
 
@@ -950,6 +958,7 @@ char *file;
 {
 	if ((Xunlink(file) < 0 && errno != ENOENT) || rmtmpdir(NULL) < 0)
 		return(-1);
+
 	return(0);
 }
 
@@ -963,6 +972,7 @@ static int dormdir(path)
 char *path;
 {
 	if (isdotdir(path)) return(0);
+
 	return(Xrmdir(path));
 }
 
@@ -1025,6 +1035,7 @@ char *dir, *file;
 		_exit(1);
 	}
 #endif
+
 	return(0);
 }
 
@@ -1038,6 +1049,7 @@ int drive;
 	path[1] = drive;
 	path[2] = '\0';
 	if (mktmpdir(path) < 0) return(NULL);
+
 	return(strdup2(path));
 }
 
@@ -1092,6 +1104,7 @@ int single;
 		return(-1);
 	}
 	*dirp = tmpdir;
+
 	return(drive);
 }
 
@@ -1126,6 +1139,7 @@ char *file;
 	struct stat st;
 
 	if (Xlstat(file, &st) < 0 && errno == ENOENT) return(0);
+
 	return(1);
 }
 
@@ -1163,6 +1177,7 @@ int dos, boundary, dirsize, ofs;
 		len += ofs;
 		if (lfn) return((len / LFNENTSIZ + 1) * DOSDIRENT + DOSDIRENT);
 	}
+
 	return(DOSDIRENT);
 }
 
@@ -1178,6 +1193,7 @@ int size, dos, boundary, dirsize, ofs;
 		size -= DOSDIRENT;
 		return((size / DOSDIRENT) * LFNENTSIZ - 1);
 	}
+
 	return(DOSBODYLEN);
 }
 
@@ -1193,6 +1209,7 @@ char *from, *to;
 	from = nodospath(fpath, from);
 	to = nodospath(tpath, to);
 #endif
+
 	return(Xrename(from, to));
 }
 
@@ -1239,8 +1256,9 @@ char *tmpdir, *old;
 					else break;
 				}
 				else {
-					if ((fd = Xopen(fname,
-					O_CREAT | O_EXCL, 0600))) {
+					fd = Xopen(fname,
+						O_CREAT | O_EXCL, 0600);
+					if (fd >= 0) {
 						Xclose(fd);
 						return(fname);
 					}
@@ -1254,6 +1272,7 @@ char *tmpdir, *old;
 		}
 	}
 	free(fname);
+
 	return(NULL);
 }
 
@@ -1264,6 +1283,7 @@ char *dir;
 	struct stat st;
 
 	if (Xlstat(dir, &st) < 0) return(getblocksize(dir));
+
 	return((off_t)st.st_size);
 }
 
@@ -1273,7 +1293,7 @@ off_t bsiz;
 {
 	struct stat st;
 	u_char ch, *tmp;
-	int i, c, n, fd;
+	int i, n, fd;
 
 	if (Xlstat(dir, &st) < 0
 	|| (fd = Xopen(dir, O_BINARY | O_RDONLY, 0666)) < 0)
@@ -1286,9 +1306,7 @@ off_t bsiz;
 			free(tmp);
 			return(NULL);
 		}
-		while ((c = Xread(fd, (char *)&ch, sizeof(ch))) < 0
-		&& errno == EINTR);
-		if (c < 0) {
+		if (sureread(fd, &ch, sizeof(ch)) < 0) {
 			free(tmp);
 			return(NULL);
 		}
