@@ -1,7 +1,7 @@
 /*
  *	func.h
  *
- *	Function Prototype Declaration
+ *	function prototype declarations
  */
 
 #ifdef	NOERRNO
@@ -79,7 +79,11 @@ extern char *sys_errlist[];
 
 /* main.c */
 extern VOID error __P_((char *));
+extern VOID setlinecol __P_((VOID_A));
 extern VOID checkscreen __P_((int, int));
+#ifdef	SIGWINCH
+extern VOID pollscreen __P_((int));
+#endif
 extern int sigvecset __P_((int));
 extern VOID title __P_((VOID_A));
 #ifndef	_NOCUSTOMIZE
@@ -87,6 +91,17 @@ extern VOID saveorigenviron __P_((VOID_A));
 #endif
 extern int loadruncom __P_((char *, int));
 
+/* system.c */
+#ifdef	_NOORIGSHELL
+#define	dosystem		system
+#define	dopopen(c)		Xpopen(c, "r")
+#define	dopclose		Xpclose
+#else
+extern int dosystem __P_((char *));
+extern FILE *dopopen __P_((char *));
+extern int dopclose __P_((FILE *));
+#endif
+
 /* dosemu.c or unixemu.c */
 #ifdef	_USEDOSPATH
 extern int _dospath __P_((char *));
@@ -137,9 +152,9 @@ extern int Xread __P_((int, char *, int));
 extern int Xwrite __P_((int, char *, int));
 extern off_t Xlseek __P_((int, off_t, int));
 #endif
-#if	!defined (LSI_C) && defined (_NODOSDRIVE)
-#define	Xdup		dup
-#define	Xdup2		dup2
+#ifdef	_NODOSDRIVE
+#define	Xdup		safe_dup
+#define	Xdup2		safe_dup2
 #else
 extern int Xdup __P_((int));
 extern int Xdup2 __P_((int, int));
@@ -180,12 +195,96 @@ extern int Xpclose __P_((FILE *));
 # endif
 #endif	/* _NOORIGSHELL */
 
+/* pty.c */
+#ifndef	_NOPTY
+extern int Xopenpty __P_((int *, char *, ALLOC_T));
+extern int Xlogin_tty __P_((char *, char *, char *));
+extern p_id_t Xforkpty __P_((int *, char *, char *));
+#endif
+
+/* termemu.c */
+#ifdef	_NOPTY
+#define	ptymacro		execmacro
+#define	ptyusercomm		execusercomm
+#define	ptysystem		dosystem
+#else	/* !_NOPTY */
+extern VOID regionscroll __P_((int, int, int, int, int, int));
+extern int selectpty __P_((int, int [], char [], int));
+extern VOID syncptyout __P_((VOID_A));
+extern int recvbuf __P_((int, VOID_P, int));
+extern VOID sendbuf __P_((int, VOID_P, int));
+extern int recvword __P_((int, int *));
+extern VOID sendword __P_((int, int));
+extern int recvstring __P_((int, char **));
+extern VOID sendstring __P_((int, char *));
+extern VOID sendparent __P_((int, ...));
+extern int ptymacro __P_((char *, char *, int));
+# ifdef	_NOORIGSHELL
+# define	ptyusercomm(c,a,f)	ptymacro(c, a, f | F_EVALMACRO)
+# else
+# define	ptyusercomm		ptymacro
+# endif
+#define	ptysystem(c)		ptymacro(c, NULL, F_DOSYSTEM)
+extern VOID killpty __P_((int, int *));
+extern VOID killallpty __P_((VOID_A));
+#endif	/* !_NOPTY */
+
+/* frontend.c */
+#ifdef	_NOPTY
+#define	Xttyiomode		ttyiomode
+#define	Xstdiomode		stdiomode
+#define	Xtermmode		termmode
+#define	Xputch2			putch2
+#define	Xcputs2			cputs2
+#define	Xputterm		putterm
+#define	Xputterms		putterms
+#define	Xsetscroll		setscroll
+#define	Xlocate			locate
+#define	Xtflush			tflush
+#define	Xcprintf2		cprintf2
+#define	Xcputnl			cputnl
+#define	Xkanjiputs		kanjiputs
+#define	Xchgcolor		chgcolor
+#define	Xmovecursor		movecursor
+#else	/* !_NOPTY */
+extern int waitstatus __P_((p_id_t, int, int *));
+extern VOID Xttyiomode __P_((int));
+extern VOID Xstdiomode __P_((VOID_A));
+extern int Xtermmode __P_((int));
+extern VOID Xputch2 __P_((int));
+extern VOID Xcputs2 __P_((char *));
+extern VOID Xputterm __P_((int));
+extern VOID Xputterms __P_((int));
+extern int Xsetscroll __P_((int, int));
+extern VOID Xlocate __P_((int, int));
+extern VOID Xtflush __P_((VOID_A));
+extern int Xcprintf2 __P_((CONST char *, ...));
+extern VOID Xcputnl __P_((VOID_A));
+extern int Xkanjiputs __P_((char *));
+extern VOID Xchgcolor __P_((int, int));
+extern VOID Xmovecursor __P_((int, int, int));
+extern VOID changewin __P_((int, p_id_t));
+extern VOID changewsize __P_((int, int));
+extern VOID insertwin __P_((int, int));
+extern VOID deletewin __P_((int, int));
+# ifndef	_NOKANJICONV
+extern VOID changekcode __P_((VOID_A));
+# endif
+extern int frontend __P_((VOID_A));
+#endif	/* !_NOPTY */
+
+/* backend.c */
+#ifndef	_NOPTY
+extern VOID resetptyterm __P_((int, int));
+extern int backend __P_((VOID_A));
+#endif
+
 /* libc.c */
 extern int stat2 __P_((char *, struct stat *));
 extern char *realpath2 __P_((char *, char *, int));
 extern int _chdir2 __P_((char *));
 extern int chdir2 __P_((char *));
-extern int chdir3 __P_((char *));
+extern int chdir3 __P_((char *, int));
 extern int mkdir2 __P_((char *, int));
 extern char *malloc2 __P_((ALLOC_T));
 extern char *realloc2 __P_((VOID_P, ALLOC_T));
@@ -210,7 +309,7 @@ extern sigcst_t signal2 __P_((int, sigcst_t));
 #define	signal2	signal
 #endif
 extern int system2 __P_((char *, int));
-extern FILE *popen2 __P_((char *, char *));
+extern FILE *popen2 __P_((char *));
 extern int pclose2 __P_((FILE *));
 extern char *getwd2 __P_((VOID_A));
 extern time_t time2 __P_((VOID_A));
@@ -244,7 +343,12 @@ extern int touchfile __P_((char *, struct stat *));
 extern int nodoslstat __P_((char *, struct stat *));
 #endif
 extern VOID lostcwd __P_((char *));
-extern int safewrite __P_((int, char *, int));
+#ifndef	NODIRLOOP
+extern int issamebody __P_((char *, char *));
+#endif
+#ifndef	NOSYMLINK
+extern int cpsymlink __P_((char *, char *));
+#endif
 extern int safecpfile __P_((char *, char *, struct stat *, struct stat *));
 extern int safemvfile __P_((char *, char *, struct stat *, struct stat *));
 extern char *genrandname __P_((char *, int));
@@ -279,7 +383,7 @@ extern int forcemovefile __P_((char *));
 
 /* parse.c */
 extern char *skipspace __P_((char *));
-extern char *evalnumeric __P_((char *, long *, int));
+extern char *sscanf2 __P_((char *, CONST char *, ...));
 #ifdef	_NOORIGSHELL
 extern char *strtkchr __P_((char *, int, int));
 extern int getargs __P_((char *, char ***));
@@ -298,7 +402,7 @@ extern VOID adjustpath __P_((VOID_A));
 #endif
 extern char *includepath __P_((char *, char *));
 #if	(FD < 2) && !defined (_NOARCHIVE)
-extern char *getrange __P_((char *, u_char *, u_char *, u_char *));
+extern char *getrange __P_((char *, int, u_char *, u_char *, u_char *));
 #endif
 extern int evalprompt __P_((char **, char *));
 #ifndef	_NOARCHIVE
@@ -317,10 +421,14 @@ extern char *encodestr __P_((char *, int));
 extern VOID freelaunch __P_((launchtable *));
 extern int searchlaunch __P_((launchtable *, int, launchtable *));
 extern int parselaunch __P_((int, char *[], launchtable *));
-extern VOID printlaunchcomm __P_((launchtable *, int, int, int, FILE *));
+extern VOID addlaunch __P_((int, launchtable *));
+extern VOID deletelaunch __P_((int));
 extern VOID freearch __P_((archivetable *));
 extern int searcharch __P_((archivetable *, int, archivetable *));
 extern int parsearch __P_((int, char *[], archivetable *));
+extern VOID addarch __P_((int, archivetable *));
+extern VOID deletearch __P_((int));
+extern VOID printlaunchcomm __P_((launchtable *, int, int, int, FILE *));
 extern VOID printarchcomm __P_((archivetable *, int, int, FILE *));
 # ifndef	_NOBROWSE
 extern VOID freebrowse __P_((launchtable *));
@@ -330,6 +438,8 @@ extern int ismacro __P_((int));
 extern int freemacro __P_((int));
 extern int searchkeybind __P_((bindtable *, bindtable *));
 extern int parsekeybind __P_((int, char *[], bindtable *));
+extern int addkeybind __P_((int, bindtable *, char *, char *, char *));
+extern VOID deletekeybind __P_((int));
 extern char *gethelp __P_((bindtable *));
 extern VOID printmacro __P_((bindtable *, int, int, FILE *));
 #ifdef	_USEDOSEMU
@@ -344,15 +454,21 @@ extern int parsekeymap __P_((int, char *[], keyseq_t *));
 extern VOID printkeymap __P_((keyseq_t *, int, FILE *));
 #endif
 #ifdef	_NOORIGSHELL
+extern int searchalias __P_((char *, int));
+extern int addalias __P_((char *, char *));
+extern int deletealias __P_((char *));
 extern VOID printalias __P_((int, FILE *));
+extern int searchfunction __P_((char *));
+extern int addfunction __P_((char *, char **));
+extern int deletefunction __P_((char *));
 extern VOID printfunction __P_((int, int, FILE *));
-#endif
+#endif	/* _NOORIGSHELL */
 extern int checkbuiltin __P_((char *));
 extern int checkinternal __P_((char *));
 extern int execbuiltin __P_((int, int, char *[]));
 extern int execinternal __P_((int, int, char *[]));
 #ifdef	_NOORIGSHELL
-extern int execpseudoshell __P_((char *, int, int));
+extern int execpseudoshell __P_((char *, int));
 #endif
 #ifndef	_NOCOMPLETE
 extern int completebuiltin __P_((char *, int, int, char ***));
@@ -395,23 +511,24 @@ extern VOID freedefine __P_((VOID_A));
 
 /* shell.c */
 extern char *restorearg __P_((char *));
-extern char *evalcommand __P_((char *, char *, macrostat *, int));
+extern char *evalcommand __P_((char *, char *, macrostat *));
 extern int replaceargs __P_((int *, char ***, char **, int));
 extern int replacearg __P_((char **));
 extern VOID demacroarg __P_((char **));
 extern char *inputshellstr __P_((char *, int, char *));
 extern char *inputshellloop __P_((int, char *));
 extern int isinternalcomm __P_((char *));
-extern int execmacro __P_((char *, char *, int, int, int));
+extern int execmacro __P_((char *, char *, int));
 extern FILE *popenmacro __P_((char *, char *, int));
 #ifdef	_NOORIGSHELL
-extern int execusercomm __P_((char *, char *, int, int, int));
+extern int execusercomm __P_((char *, char *, int));
 #else
 #define	execusercomm	execmacro
 #endif
 extern int entryhist __P_((int, char *, int));
 extern char *removehist __P_((int));
 extern int loadhistory __P_((int, char *));
+extern VOID convhistory __P_((char *, FILE *));
 extern int savehistory __P_((int, char *));
 extern int parsehist __P_((char *, int *));
 extern char *evalhistory __P_((char *));
@@ -442,9 +559,8 @@ extern u_int cnvunicode __P_((u_int, int));
 #endif
 #ifndef	_NOKANJICONV
 extern int kanjiconv __P_((char *, char *, int, int, int, int));
-# ifndef	_NOKANJIFCONV
-extern char *kanjiconv2 __P_((char *, char *, int, int, int));
-# endif
+extern char *kanjiconv2 __P_((char *, char *, int, int, int, int));
+extern char *newkanjiconv __P_((char *, int, int, int));
 #endif
 #ifndef	_NOKANJIFCONV
 extern int getkcode __P_((char *));
@@ -522,11 +638,14 @@ extern archivetable *copyarch __P_((archivetable *, archivetable *,
 extern VOID freedosdrive __P_((devinfo *));
 extern devinfo *copydosdrive __P_((devinfo *, devinfo *));
 # endif
-extern VOID rewritecust __P_((int));
+extern VOID rewritecust __P_((VOID_A));
 extern int customize __P_((VOID_A));
 #endif	/* !_NOCUSTOMIZE */
 
 /* command.c */
+#ifndef	_NOSPLITWIN
+extern int nextwin __P_((VOID_A));
+#endif
 
 /* browse.c */
 extern VOID helpbar __P_((VOID_A));
@@ -535,13 +654,15 @@ extern int putmode __P_((char *, u_int, int));
 extern int putflags __P_((char *, u_long));
 #endif
 extern VOID waitmes __P_((VOID_A));
+extern int filetop __P_((int));
 extern int calcwidth __P_((VOID_A));
 extern int putname __P_((namelist *, int, int));
-extern int listupfile __P_((namelist *, int, char *));
+extern int listupfile __P_((namelist *, int, char *, int));
 #ifndef	_NOSPLITWIN
 extern int shutwin __P_((int));
 #endif
+extern VOID calcwin __P_((VOID_A));
 extern VOID movepos __P_((int, int));
 extern VOID rewritefile __P_((int));
 extern VOID addlist __P_((VOID_A));
-extern VOID main_fd __P_((char *));
+extern VOID main_fd __P_((char **));
