@@ -4,8 +4,6 @@
  *	by T.Shirai <shirai@unixusers.net>
  */
 
-#include <ctype.h>
-#include <signal.h>
 #include "fd.h"
 #include "func.h"
 #include "kanji.h"
@@ -25,9 +23,13 @@ extern unsigned _stklen = 0x5800;
 #endif	/* !MSDOS */
 
 #ifdef	_NOORIGSHELL
+#include "termio.h"
+#include "wait.h"
 #define	Xexit2		exit2
+#define	isorgpid()	(1)
 #else
 #include "system.h"
+#define	isorgpid()	(mypid == orgpid)
 #endif
 
 #if	MSDOS
@@ -54,11 +56,10 @@ extern int maxlaunch;
 extern archivetable archivelist[];
 extern int maxarchive;
 #endif
+extern int wheader;
 extern char fullpath[];
 extern char *histfile;
 extern char *helpindex[];
-extern int tradlayout;
-extern int sizeinfo;
 extern int subwindow;
 extern int win_x;
 extern int win_y;
@@ -66,35 +67,16 @@ extern int win_y;
 extern int custno;
 #endif
 extern char *deftmpdir;
-#ifndef	_NODOSDRIVE
+#if	!defined (_NOKANJICONV) || !defined (_NODOSDRIVE)
 extern char *unitblpath;
 #endif
+#ifndef	_NOPTY
+extern int ptymode;
+extern int parentfd;
+#endif	/* !_NOPTY */
 
 #define	CLOCKUPDATE	10	/* sec */
 
-#ifndef	SIG_DFL
-#define	SIG_DFL		((sigcst_t)0)
-#endif
-#ifndef	SIG_IGN
-#define	SIG_IGN		((sigcst_t)1)
-#endif
-
-#if	!MSDOS && !defined (SIGWINCH)
-# if	defined (SIGWINDOW)
-# define	SIGWINCH	SIGWINDOW
-# else
-#  if	defined (NSIG)
-#  define	SIGWINCH	NSIG
-#  else
-#  define	SIGWINCH	28
-#  endif
-# endif
-#endif	/* !MSDOS && !SIGWINCH */
-
-#if	!defined (SIGIOT) && defined (SIGABRT)
-#define	SIGIOT	SIGABRT
-#endif
-
 #if	MSDOS && !defined (PROTECTED_MODE)
 static harderr_t far criticalerror __P_((u_int, u_int, u_short far *));
 #endif
@@ -153,11 +135,16 @@ static int xcpuerror __P_((VOID_A));
 #ifdef	SIGXFSZ
 static int xsizerror __P_((VOID_A));
 #endif
+static int NEAR minrow __P_((int));
+#ifndef	_NOEXTRAWIN
+static int NEAR biaswsize __P_((int, int));
+#endif
 #ifdef	SIGWINCH
 static int wintr __P_((VOID_A));
 #endif
+static VOID NEAR printtime __P_((int));
 #ifdef	SIGALRM
-static int printtime __P_((VOID_A));
+static int trapalrm __P_((VOID_A));
 #endif
 #ifdef	_NOORIGSHELL
 static int NEAR execruncomline __P_((char *, char *, int, char *));
@@ -181,7 +168,6 @@ int hideclock = 0;
 #ifdef	SIGALRM
 int noalrm = 1;
 #endif
-u_short today[3] = {0, 0, 0};
 int fd_restricted = 0;
 #ifndef	_NOCUSTOMIZE
 char **orighelpindex = NULL;
@@ -206,8 +192,7 @@ int fdmode = 0;
 
 static int timersec = 0;
 #ifdef	SIGWINCH
-static int winchok = 0;
-static int winched = 0;
+static int nowinch = 0;
 #endif
 
 
@@ -229,13 +214,16 @@ char *s;
 
 	duperrno = errno;
 	sigvecset(0);
-	forcecleandir(deftmpdir, tmpfilename);
+	if (isorgpid()) {
+		forcecleandir(deftmpdir, tmpfilename);
 #ifndef	_NODOSDRIVE
-	dosallclose();
+		dosallclose();
 #endif
+		Xstdiomode();
+		endterm();
+	}
+
 	if (!s) s = progname;
-	stdiomode();
-	endterm();
 	if (dumbterm <= 2) fputc('\007', stderr);
 	errno = duperrno;
 	if (errno) perror2(s);
@@ -243,26 +231,28 @@ char *s;
 		fputs(s, stderr);
 		fputnl(stderr);
 	}
-	inittty(1);
-	keyflush();
-	prepareexitfd();
+
+	if (isorgpid()) {
+		inittty(1);
+		keyflush();
+		prepareexitfd();
 #ifndef	_NOORIGSHELL
 # ifndef	NOJOB
-	killjob();
+		killjob();
 # endif
-	prepareexit(0);
+		prepareexit(0);
+#endif	/* !_NOORIGSHELL */
+#ifndef	_NOPTY
+		killallpty();
 #endif
 #ifdef	DEBUG
 # if	!MSDOS
-	freeterment();
+		freeterment();
 # endif
-	if (ttyout && ttyout != stderr) {
-		if (fileno(ttyout) == ttyio) ttyio = -1;
-		fclose(ttyout);
+		closetty(&ttyio, &ttyout);
+		muntrace();
+#endif	/* DEBUG */
 	}
-	if (ttyio >= 0) close(ttyio);
-	muntrace();
-#endif
 
 	exit(2);
 }
@@ -271,12 +261,33 @@ static VOID NEAR signalexit(sig)
 int sig;
 {
 	signal2(sig, SIG_IGN);
-	forcecleandir(deftmpdir, tmpfilename);
+
+	if (isorgpid()) {
+		forcecleandir(deftmpdir, tmpfilename);
 #ifndef	_NODOSDRIVE
-	dosallclose();
+		dosallclose();
 #endif
-	endterm();
-	inittty(1);
+		endterm();
+		inittty(1);
+		keyflush();
+#ifndef	_NOORIGSHELL
+# if	defined (SIGHUP) && !defined (NOJOB)
+		if (sig == SIGHUP) killjob();
+# endif
+		prepareexit(0);
+#endif	/* !_NOORIGSHELL */
+#ifndef	_NOPTY
+		killallpty();
+#endif
+#ifdef	DEBUG
+# if	!MSDOS
+		freeterment();
+# endif
+		closetty(&ttyio, &ttyout);
+		muntrace();
+#endif	/* DEBUG */
+	}
+
 	signal2(sig, SIG_DFL);
 	VOID_C kill(getpid(), sig);
 }
@@ -284,50 +295,79 @@ int sig;
 #ifdef	SIGALRM
 static int ignore_alrm(VOID_A)
 {
+	int duperrno;
+
+	duperrno = errno;
 	signal2(SIGALRM, (sigcst_t)ignore_alrm);
+	errno = duperrno;
+
 	return(0);
 }
-#endif
+#endif	/* SIGALRM */
 
 #ifdef	SIGWINCH
 static int ignore_winch(VOID_A)
 {
+	int duperrno;
+
+	duperrno = errno;
 	signal2(SIGWINCH, (sigcst_t)ignore_winch);
+	errno = duperrno;
+
 	return(0);
 }
-#endif
+#endif	/* SIGWINCH */
 
 #ifdef	SIGINT
 static int ignore_int(VOID_A)
 {
+	int duperrno;
+
+	duperrno = errno;
 	signal2(SIGINT, (sigcst_t)ignore_int);
+	errno = duperrno;
+
 	return(0);
 }
-#endif
+#endif	/* SIGINT */
 
 #ifdef	SIGQUIT
 static int ignore_quit(VOID_A)
 {
+	int duperrno;
+
+	duperrno = errno;
 	signal2(SIGQUIT, (sigcst_t)ignore_quit);
+	errno = duperrno;
+
 	return(0);
 }
-#endif
+#endif	/* SIGQUIT */
 
 #ifdef	SIGCONT
 static int ignore_cont(VOID_A)
 {
+	int duperrno;
+
+	duperrno = errno;
 	signal2(SIGCONT, (sigcst_t)ignore_cont);
 # if	!MSDOS
 	suspended = 1;
-# endif
+#  ifndef	_NOORIGSHELL
+	sigconted = 1;
+#  endif
+# endif	/* !MSDOS */
+	errno = duperrno;
+
 	return(0);
 }
-#endif
+#endif	/* SIGCONT */
 
 #ifdef	SIGHUP
 static int hanguperror(VOID_A)
 {
 	signalexit(SIGHUP);
+
 	return(0);
 }
 #endif
@@ -336,6 +376,7 @@ static int hanguperror(VOID_A)
 static int illerror(VOID_A)
 {
 	signalexit(SIGILL);
+
 	return(0);
 }
 #endif
@@ -344,6 +385,7 @@ static int illerror(VOID_A)
 static int traperror(VOID_A)
 {
 	signalexit(SIGTRAP);
+
 	return(0);
 }
 #endif
@@ -352,6 +394,7 @@ static int traperror(VOID_A)
 static int ioerror(VOID_A)
 {
 	signalexit(SIGIOT);
+
 	return(0);
 }
 #endif
@@ -360,6 +403,7 @@ static int ioerror(VOID_A)
 static int emuerror(VOID_A)
 {
 	signalexit(SIGEMT);
+
 	return(0);
 }
 #endif
@@ -368,6 +412,7 @@ static int emuerror(VOID_A)
 static int floaterror(VOID_A)
 {
 	signalexit(SIGFPE);
+
 	return(0);
 }
 #endif
@@ -376,6 +421,7 @@ static int floaterror(VOID_A)
 static int buserror(VOID_A)
 {
 	signalexit(SIGBUS);
+
 	return(0);
 }
 #endif
@@ -384,6 +430,7 @@ static int buserror(VOID_A)
 static int segerror(VOID_A)
 {
 	signalexit(SIGSEGV);
+
 	return(0);
 }
 #endif
@@ -392,6 +439,7 @@ static int segerror(VOID_A)
 static int syserror(VOID_A)
 {
 	signalexit(SIGSYS);
+
 	return(0);
 }
 #endif
@@ -400,6 +448,7 @@ static int syserror(VOID_A)
 static int pipeerror(VOID_A)
 {
 	signalexit(SIGPIPE);
+
 	return(0);
 }
 #endif
@@ -408,6 +457,7 @@ static int pipeerror(VOID_A)
 static int terminate(VOID_A)
 {
 	signalexit(SIGTERM);
+
 	return(0);
 }
 #endif
@@ -416,6 +466,7 @@ static int terminate(VOID_A)
 static int xcpuerror(VOID_A)
 {
 	signalexit(SIGXCPU);
+
 	return(0);
 }
 #endif
@@ -424,143 +475,233 @@ static int xcpuerror(VOID_A)
 static int xsizerror(VOID_A)
 {
 	signalexit(SIGXFSZ);
+
 	return(0);
 }
 #endif
 
+VOID setlinecol(VOID_A)
+{
+	char buf[MAXLONGWIDTH + 1];
+
+	if (getconstvar("LINES")) {
+		snprintf2(buf, sizeof(buf), "%d", n_line);
+		setenv2("LINES", buf, 1);
+	}
+	if (getconstvar("COLUMNS")) {
+		snprintf2(buf, sizeof(buf), "%d", n_column);
+		setenv2("COLUMNS", buf, 1);
+	}
+}
+
+static int NEAR minrow(n)
+int n;
+{
+	if (n == win) {
+#ifndef	_NOTREE
+		if (treepath) return(WFILEMINTREE);
+#endif
+#ifndef	_NOCUSTOMIZE
+		if (custno >= 0) return(WFILEMINCUSTOM);
+#endif
+	}
+
+	return(WFILEMIN);
+}
+
+#ifndef	_NOEXTRAWIN
+static int NEAR biaswsize(n, max)
+int n, max;
+{
+	int i, row, min;
+
+	for (i = row = 0; i < n; i++) row += winvar[i].v_fileperrow + 1;
+	min = minrow(n);
+	if (max - row < min) {
+		winvar[n].v_fileperrow = min;
+		return(max - min);
+	}
+	winvar[n].v_fileperrow = max - row;
+
+	return(0);
+}
+#endif	/* !_NOEXTRAWIN */
+
 VOID checkscreen(xmax, ymax)
 int xmax, ymax;
 {
-#ifdef	SIGWINCH
-	int dupwinchok;
-#endif
-	char *cp;
-	int i, row;
+	char *cp, *tty;
+	int i, row, wastty, dupn_line, dupdumbterm;
 
 #ifdef	SIGWINCH
-	dupwinchok = winchok;
-	winchok = 0;
+	nowinch++;
 #endif
+	dupn_line = -1;
+#ifndef	_NOPTY
+	if (parentfd >= 0) row = WFILEMIN;
+	else
+#endif
+	{
+		row = wheader + WFOOTER;
+#ifdef	_NOEXTRAWIN
+		row += (minrow(win) + 1) * windows;
+#else
+		for (i = 0; i < windows; i++) row += minrow(i) + 1;
+#endif
+		row--;
+	}
+
+	savetermio(ttyio, &tty, NULL);
+	wastty = isttyiomode;
+	Xttyiomode(1);
+	dupdumbterm = dumbterm;
+	if (xmax < 0 || ymax < 0) {
+		dupn_line = n_line;
+		xmax = WCOLUMNMIN;
+		ymax = 0;
+		dumbterm = 1;
+	}
 	for (i = 0;; i++) {
 		if (!(cp = getwsize(xmax, ymax))) {
-#ifndef	_NOTREE
-			if (treepath) row = WFILEMINTREE;
-			else
-#endif
-#ifndef	_NOCUSTOMIZE
-			if (custno >= 0) row = WFILEMINCUSTOM;
-			else
-#endif
-			row = WFILEMIN;
-
-			if (FILEPERROW >= row) break;
+			if (n_line >= row) break;
 			cp = NOROW_K;
 		}
 		if (!i) {
-			putterm(t_clear);
-			locate(0, 0);
+			Xputterms(T_CLEAR);
+			Xlocate(0, 0);
 			keyflush();
 		}
 
-		if (i & 1) cputs2(SCRSZ_K);
+		if (i & 1) Xcputs2(SCRSZ_K);
 		else {
-			putterm(t_standout);
-			cputs2(cp);
-			putterm(end_standout);
+			Xputterm(T_STANDOUT);
+			Xcputs2(cp);
+			Xputterm(END_STANDOUT);
 		}
-		putterm(t_bell);
-		cputnl();
-		keyflush();
+		Xputterm(T_BELL);
+		Xcputnl();
+		Xtflush();
 		if (kbhit2(1000000L) && getkey2(0) == K_ESC) {
 			errno = 0;
 			error(INTR_K);
 		}
 	}
+	dumbterm = dupdumbterm;
+	loadtermio(ttyio, tty, NULL);
+	isttyiomode = wastty;
+	if (tty) free(tty);
+
+	setlinecol();
+	if (n_line != dupn_line) {
+#ifdef	_NOEXTRAWIN
+		calcwin();
+#else
+		row = fileperrow(1);
+		for (i = windows - 1; i >= 0; i--) {
+			if (!(row = biaswsize(i, row))) break;
+			row--;
+		}
+#endif
+	}
 #ifdef	SIGWINCH
-	winchok = dupwinchok;
+	nowinch--;
 #endif
 }
 
 #ifdef	SIGWINCH
+VOID pollscreen(forced)
+int forced;
+{
+	static int winched = 0;
+	int x, y;
+
+	if (forced < 0) nowinch = winched = 0;
+	else if (nowinch) {
+		if (forced) winched++;
+	}
+	else if (winched || forced) {
+		winched = 0;
+		x = n_column;
+		y = n_line;
+		checkscreen(-1, -1);
+		if (isorgpid()) {
+			if (x != n_column || y != n_line) rewritefile(1);
+			if (subwindow) ungetch2(K_CTRL('L'));
+		}
+	}
+}
+
 static int wintr(VOID_A)
 {
-	int duperrno, dupusegetcursor;
+	int duperrno;
 
 	duperrno = errno;
 	signal2(SIGWINCH, SIG_IGN);
-	if (!winchok) winched++;
-	else {
-		dupusegetcursor = usegetcursor;
-		usegetcursor = 0;
-		checkscreen(WCOLUMNMIN, WHEADERMAX + WFOOTER + WFILEMIN);
-		usegetcursor = dupusegetcursor;
-		rewritefile(1);
-		if (subwindow) ungetch2(K_CTRL('L'));
-	}
+	pollscreen(1);
 	signal2(SIGWINCH, (sigcst_t)wintr);
 	errno = duperrno;
+
 	return(0);
 }
-#endif
+#endif	/* SIGWINCH */
 
-#ifdef	SIGALRM
-static int printtime(VOID_A)
+static VOID NEAR printtime(hide)
+int hide;
 {
 	static time_t now;
 	struct tm *tm;
-	int x, duperrno;
+	int x;
 
-	duperrno = errno;
-	signal2(SIGALRM, SIG_IGN);
 	if (timersec) now++;
 	else {
 		now = time2();
 		timersec = CLOCKUPDATE;
 	}
-	if (showsecond || timersec == CLOCKUPDATE) {
-# ifdef	DEBUG
-		_mtrace_file = "localtime(start)";
-		tm = localtime(&now);
-		if (_mtrace_file) _mtrace_file = NULL;
-		else {
-			_mtrace_file = "localtime(end)";
-			malloc(0);	/* dummy malloc */
-		}
-# else
-		tm = localtime(&now);
-# endif
-		today[0] = tm -> tm_year;
-		today[1] = tm -> tm_mon;
-		today[2] = tm -> tm_mday;
-		if (!hideclock) {
-			x = n_column - 15 - ((showsecond) ? 3 : 0);
-			if (!isleftshift()) x--;
-			locate(x, L_TITLE);
-			putterm(t_standout);
-			cprintf2("%02d-%02d-%02d %02d:%02d",
-				tm -> tm_year % 100,
-				tm -> tm_mon + 1, tm -> tm_mday,
-				tm -> tm_hour, tm -> tm_min);
-			if (showsecond) cprintf2(":%02d", tm -> tm_sec);
-			putterm(end_standout);
-			locate(win_x, win_y);
-			tflush();
-		}
+	if (timersec-- < CLOCKUPDATE && !showsecond) return;
+#ifndef	_NOPTY
+	if (ptymode && hide <= 1) hide = 0;
+#endif
+	if (hide) return;
+
+#ifdef	DEBUG
+	_mtrace_file = "localtime(start)";
+	tm = localtime(&now);
+	if (_mtrace_file) _mtrace_file = NULL;
+	else {
+		_mtrace_file = "localtime(end)";
+		malloc(0);	/* dummy malloc */
 	}
-	timersec--;
+#else
+	tm = localtime(&now);
+#endif
+	x = n_column - 15 - ((showsecond) ? 3 : 0);
+	if (!isleftshift()) x--;
+	Xlocate(x, L_TITLE);
+	Xputterm(T_STANDOUT);
+	Xcprintf2("%02d-%02d-%02d %02d:%02d",
+		tm -> tm_year % 100,
+		tm -> tm_mon + 1, tm -> tm_mday,
+		tm -> tm_hour, tm -> tm_min);
+	if (showsecond) Xcprintf2(":%02d", tm -> tm_sec);
+	Xputterm(END_STANDOUT);
+	Xlocate(win_x, win_y);
+	Xtflush();
+}
+
+#ifdef	SIGALRM
+static int trapalrm(VOID_A)
+{
+	int duperrno;
+
+	duperrno = errno;
+	signal2(SIGALRM, SIG_IGN);
+	if (isorgpid()) printtime(hideclock);
 # ifdef	SIGWINCH
-	if (!winchok) winchok++;
-	else if (winched) {
-		signal2(SIGWINCH, SIG_IGN);
-		winched = 0;
-		checkscreen(WCOLUMNMIN, WHEADERMAX + WFOOTER + WFILEMIN);
-		rewritefile(1);
-		if (subwindow) ungetch2(K_CTRL('L'));
-		signal2(SIGWINCH, (sigcst_t)wintr);
-	}
+	pollscreen(0);
 # endif
-	signal2(SIGALRM, (sigcst_t)printtime);
+	signal2(SIGALRM, (sigcst_t)trapalrm);
 	errno = duperrno;
+
 	return(0);
 }
 #endif	/* SIGALRM */
@@ -575,14 +716,13 @@ int set;
 	if (set == old) /*EMPTY*/;
 	else if (set) {
 #ifdef	SIGALRM
-		signal2(SIGALRM, (sigcst_t)printtime);
+		signal2(SIGALRM, (sigcst_t)trapalrm);
 		noalrm = 0;
 #endif
 #ifdef	SIGTSTP
 		signal2(SIGTSTP, SIG_IGN);
 #endif
 #ifdef	SIGWINCH
-		winchok = winched = 0;
 		signal2(SIGWINCH, (sigcst_t)wintr);
 #endif
 #ifndef	_NODOSDRIVE
@@ -617,39 +757,37 @@ VOID title(VOID_A)
 	char *cp, *eol;
 	int i, len;
 
-	locate(0, L_TITLE);
-	putterm(t_standout);
+	Xlocate(0, L_TITLE);
+	Xputterm(T_STANDOUT);
 	len = 0;
 	if (!isleftshift()) {
-		putch2(' ');
+		Xputch2(' ');
 		len++;
 	}
-	cputs2(" FD");
+	Xcputs2(" FD");
 	len += 3;
 	if (!ishardomit()) {
-		cputs2("(File & Directory tool)");
+		Xcputs2("(File & Directory tool)");
 		len += 23;
 	}
-	cputs2(" Ver.");
+	Xcputs2(" Ver.");
 	len += 5;
 	cp = strchr(version, ' ');
 	while (*(++cp) == ' ');
 	if (!(eol = strchr(cp, ' '))) eol = cp + strlen(cp);
 	i = eol - cp;
-	cprintf2("%-*.*s", i, i, cp);
+	Xcprintf2("%-*.*s", i, i, cp);
 	if (distributor) {
-		putch2('#');
+		Xputch2('#');
 		i++;
 	}
-	cp = (iswellomit()) ? "" : " (c)1995-2003 T.Shirai  ";
-	cputs2(cp);
-	i = n_column - len - strlen(cp) - i;
-	while (i-- > 0) putch2(' ');
-	putterm(end_standout);
+	cp = (iswellomit()) ? "" : " (c)1995-2005 T.Shirai  ";
+	Xcputs2(cp);
+	i = n_column - len - strlen2(cp) - i;
+	while (i-- > 0) Xputch2(' ');
+	Xputterm(END_STANDOUT);
 	timersec = 0;
-#ifdef	SIGALRM
-	printtime();
-#endif
+	printtime(0);
 }
 
 #ifndef	_NOCUSTOMIZE
@@ -682,16 +820,17 @@ char *line;
 	int i;
 
 	if (!*(cp = skipspace(command))) i = 0;
-	else i = execpseudoshell(cp, 0, 1);
+	else i = execpseudoshell(cp, F_IGNORELIST | F_NOCOMLINE);
 	if (i) {
-		putterm(l_clear);
-		cprintf2("%s, line %d: %s", file, n, ILFNC_K);
-		cputnl();
-		putterm(l_clear);
-		cprintf2("\t%s", line);
-		cputnl();
+		Xputterm(L_CLEAR);
+		Xcprintf2("%s, line %d: %s", file, n, ILFNC_K);
+		Xcputnl();
+		Xputterm(L_CLEAR);
+		Xcprintf2("\t%s", line);
+		Xcputnl();
 	}
 	free(command);
+
 	return((i) ? -1 : 0);
 }
 #endif	/* _NOORIGSHELL */
@@ -716,23 +855,23 @@ int exist;
 
 		cp = malloc2(strlen(file) + strlen(tmp) + 1 + 1);
 		strcpy(strcpy2(strcpy2(cp, file), "."), tmp);
-		tmp = evalpath(cp, 1);
-		if (stat2(tmp, &st) < 0 || (st.st_mode & S_IFMT) != S_IFREG) {
+		tmp = evalpath(cp, 0);
+		if (stat2(tmp, &st) < 0 || !s_isreg(&st)) {
 			free(tmp);
 			tmp = NULL;
 		}
 	}
 	if (!tmp)
 # endif	/* !MSDOS */
-	tmp = evalpath(strdup2(file), 1);
+	tmp = evalpath(strdup2(file), 0);
 	fp = Xfopen(tmp, "r");
 	if (!fp) {
 		if (!exist) {
 			free(tmp);
 			return(0);
 		}
-		cprintf2("%s: Not found", tmp);
-		cputnl();
+		Xcprintf2("%s: Not found", tmp);
+		Xcputnl();
 		free(tmp);
 		return(-1);
 	}
@@ -778,9 +917,9 @@ int exist;
 	fclose(fp);
 	free(tmp);
 #else	/* !_NOORIGSHELL */
-	n = stdiomode();
+	if ((n = isttyiomode)) Xstdiomode();
 	er = execruncom(file, 1);
-	ttyiomode(n);
+	if (n) Xttyiomode(n - 1);
 #endif	/* !_NOORIGSHELL */
 
 	return(er ? -1 : 0);
@@ -792,34 +931,36 @@ int argc;
 char *argv[], *envp[];
 {
 	char *cp, **optv;
-	int i, j, optc;
+	int i, optc;
 
 	optc = 1;
 	optv = (char **)malloc2((argc + 1) * sizeof(char *));
 	optv[0] = argv[0];
-	optv[1] = NULL;
 
 	for (i = 1; i < argc; i++) {
-		if (argv[i][0] == '+') cp = argv[i];
+		if (argv[i][0] == '+') /*EMPTY*/;
 		else if (argv[i][0] != '-' || !argv[i][1]
-		|| (argv[i][1] == '-' && !argv[i][2])) break;
-		else for (cp = &(argv[i][1]); *cp; cp++) {
-			if (!isidentchar(*cp)
-			&& (cp == &(argv[i][1]) || !isdigit(*cp))) break;
-		}
-		if (cp <= &(argv[i][1]) || *cp != '=') {
-			optv[optc++] = argv[i];
-			optv[optc] = NULL;
-			for (j = i; j < argc; j++) argv[j] = argv[j + 1];
-			argc--;
-			i--;
+		|| (argv[i][1] == '-' && !argv[i][2]))
+			break;
+		else if (!isidentchar(argv[i][1])) /*EMPTY*/;
+		else {
+			for (cp = &(argv[i][2]); *cp; cp++)
+				if (!isidentchar2(*cp)) break;
+			if (*cp == '=') continue;
 		}
+
+		optv[optc++] = argv[i];
+		memmove((char *)&(argv[i]), (char *)&(argv[i + 1]),
+			(argc-- - i) * sizeof(char *));
+		i--;
 	}
+	optv[optc] = NULL;
 
 #ifndef	_NOORIGSHELL
 	if (initshell(optc, optv) < 0) Xexit2(RET_FAIL);
 #endif	/* !_NOORIGSHELL */
 	free(optv);
+
 	return(argc);
 }
 
@@ -830,14 +971,18 @@ char *argv[];
 	int i;
 
 	for (i = 1; argv[i]; i++) {
-		if (argv[i][0] != '-') return(i);
-		if (!argv[i][1] || (argv[i][1] == '-' && !argv[i][2]))
-			return(i + 1);
+		if (argv[i][0] != '-') break;
+		if (!argv[i][1] || (argv[i][1] == '-' && !argv[i][2])) {
+			i++;
+			break;
+		}
 		tmp = strdup2(&(argv[i][1]));
 		if ((cp = strchr(tmp, '='))) *(cp++) = '\0';
 		setenv2(tmp, cp, 0);
 		free(tmp);
 	}
+	evalenv();
+
 	return(i);
 }
 
@@ -850,6 +995,7 @@ char *s, *envp[];
 	for (i = 0; envp[i]; i++)
 		if (!strnenvcmp(envp[i], s, len) && envp[i][len] == '=')
 			return(&(envp[i][++len]));
+
 	return(NULL);
 }
 
@@ -915,6 +1061,9 @@ static VOID NEAR prepareexitfd(VOID_A)
 	}
 	free(origpath);
 	free(progname);
+#if	!defined (_NOKANJICONV) || !defined (_NODOSDRIVE)
+	free(unitblpath);
+#endif
 #ifndef	_NODOSDRIVE
 	dosallclose();
 #endif
@@ -1088,11 +1237,11 @@ char *argv[], *envp[];
 #endif
 
 	setexecpath(argv[0], envp);
-#ifndef	_NODOSDRIVE
+#if	!defined (_NOKANJICONV) || !defined (_NODOSDRIVE)
 # ifdef	DATADIR
-	unitblpath = DATADIR;
+	unitblpath = strdup2(DATADIR);
 # else
-	unitblpath = progpath;
+	unitblpath = strdup2(progpath);
 # endif
 #endif
 
@@ -1100,6 +1249,7 @@ char *argv[], *envp[];
 	inittty(0);
 	getterment(NULL);
 	argc = initoption(argc, argv, envp);
+	adjustpath();
 #else	/* !_NOORIGSHELL */
 	cp = getshellname(progname, NULL, NULL);
 	if (!strpathcmp(cp, FDSHELL) || !strpathcmp(cp, "su")) {
@@ -1117,65 +1267,62 @@ char *argv[], *envp[];
 	}
 #endif	/* !_NOORIGSHELL */
 
-	ttyiomode(0);
+	Xttyiomode(0);
 	initterm();
 	if ((cp = getwsize(WCOLUMNMIN, WHEADERMAX + WFOOTER + WFILEMIN))) {
 		errno = 0;
 		error(cp);
 	}
+	setlinecol();
 #ifndef	_NOCUSTOMIZE
 	saveorigenviron();
 #endif
 
-	locate(0, n_line - 1);
+	Xlocate(0, n_line - 1);
 	inruncom = 1;
-#if	MSDOS && !defined (BSPATHDELIM)
-	/* DEFRUNCOM is gave from command line, not to convert previously */
-/*NOTDEFINED*/
-	cp = strdup2(DEFRUNCOM);
-	i = loadruncom(adjustpname(cp), 0);
-	free(cp);
-#else
-/*NOTDEFINED*/
-	i = loadruncom(DEFRUNCOM, 0);
-#endif
-	i += loadruncom(RUNCOMFILE, 0);
+	i = loadruncom(DEFRC, 0);
+	i += loadruncom(FD_RCFILE, 0);
 	inruncom = 0;
+#ifdef	SIGWINCH
+	nowinch = 1;
+#endif
 	sigvecset(1);
 	if (i < 0) {
-		hideclock = 1;
+		hideclock = 2;
 		warning(0, HITKY_K);
 	}
+
 	i = evaloption(argv);
-#ifdef	_NOORIGSHELL
-	adjustpath();
-#endif
-	evalenv();
+	checkscreen(WCOLUMNMIN, WHEADERMAX + WFOOTER + WFILEMIN);
 #if	!MSDOS
 	if (adjtty) {
-		stdiomode();
+		Xstdiomode();
 		inittty(0);
-		ttyiomode(0);
+		Xttyiomode(0);
 	}
 #endif	/* !MSDOS */
 
 	loadhistory(0, histfile);
 	entryhist(1, origpath, 1);
-	putterms(t_clear);
+	Xputterms(T_CLEAR);
 
 #ifdef	SIGWINCH
-	winchok = 1;
+	nowinch = 0;
 #endif
-	main_fd(argv[i]);
+	main_fd(&(argv[i]));
 	sigvecset(0);
 	prepareexitfd();
 
-	stdiomode();
+	Xstdiomode();
 #ifndef	_NOORIGSHELL
 # ifndef	NOJOB
 	killjob();
 # endif
+#endif	/* !_NOORIGSHELL */
+#ifndef	_NOPTY
+	killallpty();
 #endif
 	Xexit2(0);
+
 	return(0);
 }
