@@ -4,9 +4,16 @@
  *	sed script maker for Makefile
  */
 
+#define	__FD_PRIMAL__
 #include "fd.h"
-#if	!MSDOS
 #include <sys/param.h>
+
+#ifndef	NOUNISTDH
+#include <unistd.h>
+#endif
+
+#ifndef	NOSTDLIBH
+#include <stdlib.h>
 #endif
 
 #ifdef	USESELECTH
@@ -22,15 +29,27 @@
 #  ifdef	OPEN_MAX
 #  define	MAXOPENFILE	OPEN_MAX
 #  else
-#   if	MSDOS
-#   define	MAXOPENFILE	20
-#   else
-#   define	MAXOPENFILE	64
-#   endif
+#  define	MAXOPENFILE	64
 #  endif
 # endif
 #endif
 
+#ifndef	VER
+#define	VER		"0.00"
+#endif
+#ifndef	DIST
+#define	DIST		""
+#endif
+#ifndef	PREFIX
+#define	PREFIX		"/usr/local"
+#endif
+#ifndef	CONFDIR
+#define	CONFDIR		"/etc"
+#endif
+#ifndef	CCCOMMAND
+#define	CCCOMMAND	"cc"
+#endif
+
 
 /*ARGSUSED*/
 int main(argc, argv)
@@ -42,15 +61,21 @@ char *argv[];
 #endif
 
 	printf("s:__VERSION__:%d:\n", FD);
-/*NOTDEFINED*/
 #if	FD >= 2
 	printf("s:__RCVERSION__:%d:\n", FD);
 #else
 	printf("s:__RCVERSION__::\n");
 #endif
-/*NOTDEFINED*/
+	printf("s:__VER__:%s:\n", VER);
+	if ((int)sizeof(DIST) > 1) printf("s:__DIST__:%s:\n", DIST);
+	else printf("s:__DIST__:none:\n");
 	printf("s:__PREFIX__:%s:\n", PREFIX);
+	printf("s:__CONFDIR__:%s:\n", CONFDIR);
+#ifdef	__CYGWIN__
+	printf("s:__EXE__:.exe:g\n");
+#else
 	printf("s:__EXE__::g\n");
+#endif
 	printf("s:__OBJ__:.o:g\n");
 	printf("s:__OBJS__:dosemu.o:\n");
 	printf("s:__OBJLIST__:$(OBJ1) $(OBJ2) $(OBJ3):\n");
