@@ -1,7 +1,7 @@
 /*
  *	pathname.c
  *
- *	Path Name Management Module
+ *	path name management module
  */
 
 #ifdef	FD
@@ -183,7 +183,7 @@ static int NEAR completefile __P_((char *, int, int, char ***,
 	char *, int, int));
 static int NEAR completeexe __P_((char *, int, int, char ***));
 #endif	/* !FDSH && !_NOCOMPLETE */
-static int NEAR addmeta __P_((char *, char *, int, int));
+static int NEAR addmeta __P_((char *, char *, int));
 static int NEAR skipvar __P_((char **, int *, int *, int));
 #ifdef	MINIMUMSHELL
 static int NEAR skipvarvalue __P_((char *, int *, char *, int, int));
@@ -246,6 +246,7 @@ char *s;
 	fputs(s, stderr);
 	fputs(": memory allocation error", stderr);
 	fputnl(stderr);
+
 	exit(2);
 }
 
@@ -260,6 +261,7 @@ ALLOC_T size;
 		tmp = NULL;	/* fake for -Wuninitialized */
 #endif
 	}
+
 	return(tmp);
 }
 
@@ -277,6 +279,7 @@ ALLOC_T size;
 		tmp = NULL;	/* fake for -Wuninitialized */
 #endif
 	}
+
 	return(tmp);
 }
 
@@ -289,6 +292,7 @@ ALLOC_T n, *sizep;
 		return(malloc2(*sizep));
 	}
 	while (n + 1 >= *sizep) *sizep *= 2;
+
 	return(realloc2(ptr, *sizep));
 }
 
@@ -302,6 +306,7 @@ char *s;
 	n = strlen(s);
 	if (!(tmp = (char *)malloc((ALLOC_T)n + 1))) error("malloc()");
 	memcpy(tmp, s, n + 1);
+
 	return(tmp);
 }
 
@@ -317,6 +322,7 @@ int n;
 	if (!(tmp = (char *)malloc((ALLOC_T)i + 1))) error("malloc()");
 	memcpy(tmp, s, i);
 	tmp[i] = '\0';
+
 	return(tmp);
 }
 
@@ -333,6 +339,7 @@ int c;
 		else if (isekana(s, i)) i++;
 #endif
 	}
+
 	return(NULL);
 }
 
@@ -343,6 +350,7 @@ char *s1, *s2;
 
 	for (i = 0; s2[i]; i++) s1[i] = s2[i];
 	s1[i] = '\0';
+
 	return(&(s1[i]));
 }
 
@@ -354,6 +362,7 @@ int n;
 
 	for (i = 0; i < n && s2[i]; i++) s1[i] = s2[i];
 	s1[i] = '\0';
+
 	return(&(s1[i]));
 }
 
@@ -395,6 +404,7 @@ char *dir;
 			return(NULL);
 		}
 	}
+
 	return(dirp);
 }
 
@@ -404,6 +414,7 @@ DIR *dirp;
 	free(dirp -> dd_buf);
 	free(dirp -> dd_path);
 	free(dirp);
+
 	return(0);
 }
 
@@ -477,6 +488,7 @@ struct stat *stp;
 	stp -> st_dev = stp -> st_ino = 0;
 	stp -> st_nlink = 1;
 	stp -> st_uid = stp -> st_gid = -1;
+
 	return(0);
 }
 # endif	/* MSDOS */
@@ -497,6 +509,7 @@ struct stat *stp;
 		}
 		stp -> st_mode &= ~S_IFMT;
 	}
+
 	return(0);
 }
 # endif	/* !NOSYMLINK */
@@ -510,6 +523,7 @@ char *path;
 
 	if (!(cp = (char *)getcwd(path, MAXPATHLEN))) return(NULL);
 	for (i = 0; cp[i]; i++) if (cp[i] == '/') cp[i] = _SC_;
+
 	return(cp);
 }
 # endif	/* !DJGPP */
@@ -529,6 +543,7 @@ int d;
 		if (iskanji1(s, i)) i++;
 # endif
 	}
+
 	return(NULL);
 }
 
@@ -547,6 +562,7 @@ int d;
 		if (iskanji1(s, i)) i++;
 # endif
 	}
+
 	return(cp);
 }
 #endif	/* MSDOS || (FD && !_NODOSDRIVE) */
@@ -563,9 +579,11 @@ char *s, *eol;
 		if (s[i] == _SC_) cp = &(s[i]);
 		if (iskanji1(s, i)) i++;
 	}
+
 	return(cp);
 #else
 	for (eol--; eol >= s; eol--) if (*eol == _SC_) return(eol);
+
 	return(NULL);
 #endif
 }
@@ -585,6 +603,7 @@ int ptr;
 
 	for (i = 0; s[i] && i < ptr; i++) if (iskanji1(s, i)) i++;
 	if (!s[i] || i > ptr) return(1);
+
 	return(!iskanji1(s, i));
 #else
 	return(1);
@@ -622,6 +641,7 @@ char *s;
 		*cp = _SC_;
 	}
 	*(++cp) = '\0';
+
 	return(cp);
 }
 
@@ -675,6 +695,7 @@ char *buf, *s1, *s2;
 		for (i = 0; s2[i] && i < len; i++) *(cp++) = s2[i];
 	}
 	*cp = '\0';
+
 	return(cp);
 }
 
@@ -694,6 +715,7 @@ char *s1, *s2;
 		s1++;
 		s2++;
 	}
+
 	return(0);
 }
 
@@ -715,6 +737,7 @@ int n;
 		s1++;
 		s2++;
 	}
+
 	return(0);
 }
 
@@ -723,7 +746,8 @@ int strpathcmp2(s1, s2)
 char *s1, *s2;
 {
 	if (pathignorecase) return(strcasecmp2(s1, s2));
-	else return(strcmp(s1, s2));
+
+	return(strcmp(s1, s2));
 }
 
 int strnpathcmp2(s1, s2, n)
@@ -731,7 +755,8 @@ char *s1, *s2;
 int n;
 {
 	if (pathignorecase) return(strncasecmp2(s1, s2, n));
-	else return(strncmp(s1, s2, n));
+
+	return(strncmp(s1, s2, n));
 }
 #endif	/* !PATHNOCASE */
 
@@ -765,6 +790,7 @@ int len;
 	cp = strndup2(ident, len);
 	env = getenv2(cp);
 	free(cp);
+
 	return(env);
 #endif
 }
@@ -788,6 +814,7 @@ int len;
 	ret = (putvarfunc) ? (*putvarfunc)(cp, len) : putenv(cp);
 #endif
 	if (ret < 0) free(cp);
+
 	return(ret);
 }
 
@@ -807,6 +834,7 @@ int isdotdir(s)
 char *s;
 {
 	if (s[0] == '.' && (!s[1] || (s[1] == '.' && !s[2]))) return(1);
+
 	return(0);
 }
 
@@ -817,6 +845,7 @@ char *s;
 	if (_dospath(s)) s += 2;
 #endif
 	if (*s != _SC_) return(NULL);
+
 	return(s);
 }
 
@@ -826,6 +855,7 @@ char *s;
 	char *cp;
 
 	if ((cp = strrdelim(s, 1))) return(cp + 1);
+
 	return(s);
 }
 
@@ -945,6 +975,7 @@ int len;
 	}
 	re[j++] = '$';
 	re[j++] = '\0';
+
 	return(re);
 }
 #endif	/* _NOORIGGLOB */
@@ -967,6 +998,7 @@ int len;
 	s = cnvregexp(s, len);
 	re_comp(s);
 	free(s);
+
 	return((reg_t *)1);
 }
 
@@ -977,6 +1009,7 @@ char *s;
 int fname;
 {
 	if (fname && skipdotfile && *s == '.') return(0);
+
 	return(re_exec(s) > 0);
 }
 
@@ -1008,6 +1041,7 @@ int len;
 		re = NULL;
 	}
 	free(s);
+
 	return(re);
 }
 
@@ -1017,6 +1051,7 @@ char *s;
 int fname;
 {
 	if (!re || (fname && skipdotfile && *s == '.')) return(0);
+
 	return(!regexec(re, s, 0, NULL, 0));
 }
 
@@ -1044,6 +1079,7 @@ int len;
 	s = cnvregexp(s, len);
 	re = regcmp(s, 0);
 	free(s);
+
 	return(re);
 }
 
@@ -1053,6 +1089,7 @@ char *s;
 int fname;
 {
 	if (!re || (fname && skipdotfile && *s == '.')) return(0);
+
 	return(regex(re, s) ? 1 : 0);
 }
 
@@ -1228,6 +1265,7 @@ int len;
 		re = b_realloc(re, n, reg_t);
 		re[n] = NULL;
 	}
+
 	return((reg_t *)realloc2(re, (n + 1) * sizeof(reg_t)));
 }
 
@@ -1297,6 +1335,7 @@ char *s;
 	}
 	while (re[n1] == wildsymbol2) n1++;
 	if (re[n1] || s[n2]) return(0);
+
 	return(1);
 }
 
@@ -1306,6 +1345,7 @@ char *s;
 int fname;
 {
 	if (!re || (fname && skipdotfile && *s == '.')) return(0);
+
 	return(_regexp_exec(re, s));
 }
 
@@ -1585,6 +1625,7 @@ int flags;
 
 	argv[argc] = NULL;
 	if (argc > 1) qsort(argv, argc, sizeof(char *), cmppath);
+
 	return(argv);
 }
 
@@ -1596,6 +1637,7 @@ char *s;
 	int i;
 
 	for (i = n = 0; s[i]; i++) n += (u_char)(s[i]);
+
 	return(n % MAXHASH);
 }
 
@@ -1621,6 +1663,7 @@ hashlist *next;
 	new -> hits = 0;
 	new -> cost = cost;
 	new -> next = next;
+
 	return(new);
 }
 
@@ -1667,6 +1710,7 @@ hashlist **htable;
 			nextp = &((*nextp) -> next);
 		}
 	}
+
 	return(new);
 }
 
@@ -1678,6 +1722,7 @@ int n;
 
 	for (hp = hashtable[n]; hp; hp = hp -> next)
 		if (!strpathcmp(com, hp -> command)) return(hp);
+
 	return(NULL);
 }
 
@@ -1713,6 +1758,7 @@ int dirok, exe;
 	d = ((st.st_mode & S_IFMT) == S_IFDIR);
 	if (!exe) return(d);
 	if (d) return(dirok ? d : -1);
+
 	return(Xaccess(path, X_OK));
 }
 
@@ -1737,6 +1783,7 @@ int len, exe;
 		strcpy(&(path[len]), "BAT");
 		if (isexecute(path, 0, exe) >= 0) return(CM_ADDEXT | CM_BATCH);
 	}
+
 	return(-1);
 }
 #endif	/* MSDOS */
@@ -1900,6 +1947,7 @@ char **argv;
 
 	for (i = 0; i < argc; i++)
 		if (!strpathcmp(argv[i], target)) return(argv[i]);
+
 	return(NULL);
 }
 
@@ -1971,6 +2019,7 @@ char ***argvp;
 	}
 	endpwent();
 #  endif	/* !DEBUG */
+
 	return(argc);
 }
 # endif	/* !NOUID */
@@ -1984,21 +2033,23 @@ int dlen, exe;
 	DIR *dirp;
 	struct dirent *dp;
 	char *cp, *new, path[MAXPATHLEN];
-	int d, size;
+	int d, size, dirok;
 
 	if (dlen >= MAXPATHLEN - 2) return(argc);
 	strncpy2(path, dir, dlen);
 	if (!(dirp = Xopendir(path))) return(argc);
 	cp = strcatdelim(path);
 
+	dirok = (exe <= 1) ? 1 : 0;
 	while ((dp = Xreaddir(dirp))) {
 		if ((!len && isdotdir(dp -> d_name))
-		|| strnpathcmp(file, dp -> d_name, len)) continue;
+		|| strnpathcmp(file, dp -> d_name, len))
+			continue;
 		size = strlen(dp -> d_name);
 		if (size + (cp - path) >= MAXPATHLEN) continue;
 		strncpy2(cp, dp -> d_name, size);
 
-		if ((d = isexecute(path, 1, exe)) < 0) continue;
+		if ((d = isexecute(path, dirok, exe)) < 0) continue;
 
 		new = malloc2(size + 1 + 1);
 		strncpy(new, dp -> d_name, size);
@@ -2014,6 +2065,7 @@ int dlen, exe;
 		(*argvp)[argc++] = new;
 	}
 	Xclosedir(dirp);
+
 	return(argc);
 }
 
@@ -2026,7 +2078,7 @@ char ***argvp;
 	int dlen;
 
 # ifdef	CWDINPATH
-	argc = completefile(file, len, argc, argvp, ".", 1, 1);
+	argc = completefile(file, len, argc, argvp, ".", 1, 2);
 # endif
 	if (!(next = getconstvar("PATH"))) return(argc);
 	for (cp = next; cp; cp = next) {
@@ -2038,9 +2090,10 @@ char ***argvp;
 		dlen = (next) ? (next++) - cp : strlen(cp);
 		tmp = _evalpath(cp, cp + dlen, 0);
 		dlen = strlen(tmp);
-		argc = completefile(file, len, argc, argvp, tmp, dlen, 1);
+		argc = completefile(file, len, argc, argvp, tmp, dlen, 2);
 		free(tmp);
 	}
+
 	return(argc);
 }
 
@@ -2073,6 +2126,7 @@ int exe;
 # endif
 	else if (exe && dir == path)
 		return(completeexe(path, len, argc, argvp));
+
 	return(completefile(path, len, argc, argvp, ".", 1, exe));
 }
 
@@ -2095,45 +2149,34 @@ char **argv;
 		free(common);
 		return(NULL);
 	}
+
 	return(common);
 }
 #endif	/* !FDSH && !_NOCOMPLETE */
 
-static int NEAR addmeta(s1, s2, quoted, flags)
+static int NEAR addmeta(s1, s2, quoted)
 char *s1, *s2;
-int quoted, flags;
+int quoted;
 {
-	int i, j;
+	int i, j, pc, quote;
 
 	if (!s2) return(0);
 	for (i = j = 0; s2[i]; i++, j++) {
-		if (s2[i] == '"') {
+		quote = quoted;
+		pc = parsechar(&(s2[i]), -1, '\0', EA_EOLMETA, &quote, NULL);
+		if (pc == PC_OPQUOTE || pc == PC_CLQUOTE || pc == PC_META) {
 			if (s1) s1[j] = PMETA;
 			j++;
 		}
-		else if (s2[i] == PMETA) {
-			if (s1) s1[j] = PMETA;
-			j++;
-			if (!(flags & EA_KEEPMETA) && s2[i + 1] == PMETA) i++;
-		}
-		else if (!quoted && s2[i] == '\'') {
-			if (s1) s1[j] = PMETA;
-			j++;
-		}
-		else if (iskanji1(s2, i)) {
+		else if (pc == PC_WORD) {
 			if (s1) s1[j] = s2[i];
 			j++;
 			i++;
 		}
-#ifdef	CODEEUC
-		else if (isekana(s2, i)) {
-			if (s1) s1[j] = s2[i];
-			j++;
-			i++;
-		}
-#endif
+
 		if (s1) s1[j] = s2[i];
 	}
+
 	return(j);
 }
 
@@ -2154,6 +2197,7 @@ int delim;
 		if (delim) cp[len++] = delim;
 		len = strcpy2(&(cp[len]), argv[i]) - cp;
 	}
+
 	return(cp);
 }
 
@@ -2164,6 +2208,7 @@ char **var;
 
 	if (!var) return(0);
 	for (i = 0; var[i]; i++);
+
 	return(i);
 }
 
@@ -2195,6 +2240,7 @@ int margin;
 	dupl = (char **)malloc2((n + margin + 1) * sizeof(char *));
 	for (i = 0; i < n; i++) dupl[i] = strdup2(var[i]);
 	dupl[i] = NULL;
+
 	return(dupl);
 }
 
@@ -2213,22 +2259,18 @@ int len, spc, flags, *qp, *pqp;
 	else if (iskanji1(s, 0)) return(PC_WORD);
 #ifdef	CODEEUC
 	else if (isekana(s, 0)) return(PC_WORD);
-#endif
-#ifdef	NESTINGQUOTE
-	else if (*qp == '`') return(PC_BQUOTE);
 #endif
 	else if (*qp == '\'') return(PC_SQUOTE);
-	else if (spc && *s == spc) return(*s);
 	else if (ismeta(s, 0, *qp, len, flags)) return(PC_META);
+	else if (*qp == '`') return(PC_BQUOTE);
 #ifdef	NESTINGQUOTE
 	else if ((flags & EA_BACKQ) && *s == '`') {
 		if (pqp && *qp) *pqp = *qp;
 		*qp = *s;
 		return(PC_OPQUOTE);
 	}
-#else
-	else if (*qp == '`') return(PC_BQUOTE);
 #endif
+	else if (spc && *s == spc) return(*s);
 	else if (*qp) return(PC_DQUOTE);
 	else if (!(flags & EA_NOEVALQ) && *s == '\'') {
 		*qp = *s;
@@ -2336,6 +2378,7 @@ int *eolp, *ptrp, qed;
 	if (skipvarvalue(*bufp, ptrp, "}", qed, 1, 0) < 0) return(-1);
 # endif
 #endif	/* BASHSTYLE */
+
 	return(mode);
 }
 
@@ -2399,6 +2442,7 @@ int qed, nonl, nest;
 
 		(*ptrp)++;
 	}
+
 	return(-1);
 }
 
@@ -2458,6 +2502,7 @@ int plen, mode;
 		free(tmp);
 	}
 	regexp_free(re);
+
 	return(ret);
 }
 
@@ -2476,6 +2521,7 @@ int plen, mode;
 		else new[i] = strdup2(var[i]);
 	}
 	new[i] = NULL;
+
 	return(new);
 }
 #endif	/* !MINIMUMSHELL */
@@ -2524,20 +2570,18 @@ int plen, *modep;
 #endif
 
 			for (i = j = 0; arglist[i + 1]; i++)
-				j += addmeta(NULL,
-					arglist[i + 1], quoted, EA_KEEPMETA);
+				j += addmeta(NULL, arglist[i + 1], quoted);
 			if (i <= 0) cp = strdup2("");
 			else {
 				j += (i - 1) * 3;
 				cp = malloc2(j + 1);
-				j = addmeta(cp, arglist[1],
-					quoted, EA_KEEPMETA);
+				j = addmeta(cp, arglist[1], quoted);
 				for (i = 2; arglist[i]; i++) {
 					cp[j++] = quoted;
 					cp[j++] = sp;
 					cp[j++] = quoted;
 					j += addmeta(&(cp[j]), arglist[i],
-						quoted, EA_KEEPMETA);
+						quoted);
 				}
 				cp[j] = '\0';
 			}
@@ -2594,6 +2638,7 @@ int plen, *modep;
 #ifndef	MINIMUMSHELL
 	freevar(new);
 #endif
+
 	return(cp);
 }
 
@@ -2652,6 +2697,7 @@ int s, len, vlen, mode, quoted;
 		if (exitfunc) (*exitfunc)();
 		return(-2);
 	}
+
 	return(mode);
 }
 
@@ -2663,6 +2709,7 @@ int olen, nlen;
 {
 	if (nlen <= olen) return(buf);
 	buf = realloc2(buf, ptr + nlen + (int)strlen(arg) - olen + 1);
+
 	return(buf);
 }
 
@@ -2779,9 +2826,9 @@ int quoted;
 	}
 
 	if (!mode && (c != '@' || !quoted)) {
-		vlen = addmeta(NULL, cp, quoted, EA_KEEPMETA);
+		vlen = addmeta(NULL, cp, quoted);
 		*bufp = insertarg(*bufp, ptr, arg, *argp - arg + 1, vlen);
-		addmeta(&((*bufp)[ptr]), cp, quoted, EA_KEEPMETA);
+		addmeta(&((*bufp)[ptr]), cp, quoted);
 	}
 	else if (!cp) vlen = 0;
 	else if (c == '@' && !*cp && quoted
@@ -2798,6 +2845,7 @@ int quoted;
 
 	ptr += vlen;
 	if (new) free(new);
+
 	return(ptr);
 }
 
@@ -2846,6 +2894,7 @@ char *name;
 	uidlist[maxuid].uid = pwd -> pw_uid;
 	uidlist[maxuid].name = strdup2(pwd -> pw_name);
 	uidlist[maxuid].home = strdup2(pwd -> pw_dir);
+
 	return(&(uidlist[maxuid++]));
 }
 
@@ -2894,6 +2943,7 @@ char *name;
 	gidlist[maxgid].name = strdup2(grp -> gr_name);
 	gidlist[maxgid].gr_mem = duplvar(grp -> gr_mem, -1);
 	gidlist[maxgid].ismem = 0;
+
 	return(&(gidlist[maxgid++]));
 }
 
@@ -2956,12 +3006,12 @@ int rest;
 	char *tmp;
 	int len, size;
 
-	stripquote(bbuf, 0);
+	stripquote(bbuf, EA_BACKQ);
 	if (!(tmp = (*backquotefunc)(bbuf))) return(buf);
-	len = addmeta(NULL, tmp, '\0', 0);
+	len = addmeta(NULL, tmp, '\0');
 	size = *ptrp + len + rest + 1;
 	buf = realloc2(buf, size);
-	addmeta(&(buf[*ptrp]), tmp, '\0', 0);
+	addmeta(&(buf[*ptrp]), tmp, '\0');
 	*ptrp += len;
 	free(tmp);
 
@@ -2998,6 +3048,7 @@ char *gethomedir(VOID_A)
 	if (pwd) return(pwd -> pw_dir);
 # endif	/* !FD */
 #endif	/* !NOUID */
+
 	return(NULL);
 }
 
@@ -3015,7 +3066,7 @@ char **argp;
 	len = ((cp = strdelim(top, 0))) ? (cp - top) : strlen(top);
 	if (!len) cp = gethomedir();
 # ifdef	FD
-	else if (len == sizeof("FD") - 1 && !strnpathcmp(top, "FD", len))
+	else if (len == (int)sizeof("FD") - 1 && !strnpathcmp(top, "FD", len))
 		cp = progpath;
 # endif
 	else {
@@ -3062,6 +3113,7 @@ char **argp;
 	strncpy(&((*bufp)[ptr]), cp, vlen);
 	ptr += vlen;
 	*argp += len;
+
 	return(ptr);
 }
 #endif	/* !MINIMUMSHELL */
@@ -3151,7 +3203,7 @@ int qed, flags;
 # else
 			else if (cp[1] == '$' || cp[1] == '~') cp++;
 # endif
-#endif
+#endif	/* FAKEMETA */
 			else if ((i = evalvar(&buf, i, &cp, tmpq)) < 0) {
 				if (bbuf) free(bbuf);
 				free(buf);
@@ -3161,14 +3213,22 @@ int qed, flags;
 		}
 		else if (pc == PC_META) {
 			cp++;
-			if (flags & EA_KEEPMETA) buf[i++] = PMETA;
+			if (flags & EA_KEEPMETA) pc = PC_NORMAL;
 			else if (*cp == '$') /*EMPTY*/;
 			else if ((flags & EA_BACKQ) && *cp == '`') /*EMPTY*/;
 			else if ((flags & EA_STRIPQ)
 			&& (*cp == '\'' || *cp == '"'))
 				/*EMPTY*/;
-			else buf[i++] = PMETA;
-			buf[i++] = *cp;
+			else pc = PC_NORMAL;
+
+			if (q == '`') {
+				bbuf[j++] = PMETA;
+				bbuf[j++] = *cp;
+			}
+			else {
+				if (pc != PC_META) buf[i++] = PMETA;
+				buf[i++] = *cp;
+			}
 		}
 		else if (pc == PC_OPQUOTE) {
 			if (*cp == '`') j = 0;
@@ -3246,6 +3306,7 @@ char ***argvp, *ifs;
 				(argc-- - n) * sizeof(char *));
 		}
 	}
+
 	return(argc);
 }
 
@@ -3278,6 +3339,7 @@ int flags;
 		free(wild);
 		n += i - 1;
 	}
+
 	return(argc);
 }
 
@@ -3292,22 +3354,24 @@ int flags;
 	for (i = j = 0, quote = '\0'; arg[i]; i++) {
 		pc = parsechar(&(arg[i]), -1, '\0', 0, &quote, NULL);
 		if (pc == PC_OPQUOTE || pc == PC_CLQUOTE) {
-			stripped = 1;
+			stripped++;
 			if (flags & EA_STRIPQ) continue;
 		}
 		else if (pc == PC_WORD) arg[j++] = arg[i++];
 		else if (pc == PC_META) {
-			stripped = 1;
-			if ((flags & EA_KEEPMETA)
-			|| quote == '\''
-			|| (quote == '"' && !strchr(DQ_METACHAR, arg[i + 1])))
-				arg[j++] = arg[i];
 			i++;
+			if (flags & EA_KEEPMETA) pc = PC_NORMAL;
+			else if (!quote && !(flags & EA_BACKQ)) /*EMPTY*/;
+			else if (!strchr(DQ_METACHAR, arg[i])) pc = PC_NORMAL;
+
+			if (pc != PC_META) arg[j++] = PMETA;
+			else stripped++;
 		}
 
 		arg[j++] = arg[i];
 	}
 	arg[j] = '\0';
+
 	return(stripped);
 }
 
@@ -3360,8 +3424,7 @@ int flags;
 		else if (pc == PC_META) {
 			i++;
 			if ((flags & EA_KEEPMETA)
-			|| quote == '\''
-			|| (quote == '"' && !strchr(DQ_METACHAR, cp[i])))
+			|| (quote && !strchr(DQ_METACHAR, cp[i])))
 				tmp[j++] = PMETA;
 		}
 		else if (pc == PC_OPQUOTE) {
@@ -3378,6 +3441,7 @@ int flags;
 	}
 	tmp[j] = '\0';
 	free(cp);
+
 	return(tmp);
 }
 
@@ -3391,5 +3455,6 @@ int flags;
 	for (cp = path; *cp == ' ' || *cp == '\t'; cp++);
 	cp = _evalpath(cp, NULL, flags);
 	free(path);
+
 	return(cp);
 }
