@@ -1,7 +1,7 @@
 /*
  *	pathname.h
  *
- *	Function Prototype Declaration for "pathname.c"
+ *	function prototype declarations for "pathname.c"
  */
 
 #ifndef	__PATHNAME_H_
@@ -13,6 +13,14 @@
 #include <sys/stat.h>
 #endif
 
+/* #define BASHSTYLE		; rather near to bash style */
+/* #define MINIMUMSHELL		; omit verbose extension from Bourne shell */
+/* #define NESTINGQUOTE		; allow `...` included in "..." */
+
+#ifndef	MINIMUMSHELL
+#define	NESTINGQUOTE
+#endif
+
 #define	IFS_SET		" \t\n"
 #define	META		'\\'
 #if	MSDOS && defined (_NOORIGSHELL)
@@ -24,19 +32,24 @@
 #ifndef	BSPATHDELIM
 #define	PMETA		META
 #define	RMSUFFIX	'%'
-#define	METACHAR	"\t\n !\"#$&'()*;<=>?[\\]`|"
+# ifdef	BASHSTYLE
+	/* bash treats '\r' as just a character */
+# define	METACHAR	"\t\n !\"#$&'()*;<=>?[\\]`|"
+# else
+# define	METACHAR	"\t\r\n !\"#$&'()*;<=>?[\\]`|"
+# endif
 #define	DQ_METACHAR	"\"$\\`"
 #else	/* BSPATHDELIM */
 # ifdef	_NOORIGSHELL
 # define	FAKEMETA
 # define	PMETA		'$'
 # define	RMSUFFIX	'%'
-# define	METACHAR	"\t\n !\"$'*<>?|"
+# define	METACHAR	"\t\r\n !\"$'*<>?|"
 # define	DQ_METACHAR	"\"$"
 # else
 # define	PMETA		'%'
 # define	RMSUFFIX	'\\'
-# define	METACHAR	"\t\n !\"#$%&'()*;<=>?[]`|"
+# define	METACHAR	"\t\r\n !\"#$%&'()*;<=>?[]`|"
 # define	DQ_METACHAR	"\"$%`"
 # endif
 #endif	/* BSPATHDELIM */
@@ -50,17 +63,6 @@
 #define	PC_WORD		6
 #define	PC_META		7
 
-#define	MAXLONGWIDTH	20		/* log10(2^64) = 19.266 */
-#ifdef	LSI_C
-#define	toupper2	toupper
-#define	tolower2	tolower
-#else
-extern u_char uppercase[256];
-extern u_char lowercase[256];
-#define	toupper2(c)	(uppercase[(u_char)(c)])
-#define	tolower2(c)	(lowercase[(u_char)(c)])
-#endif
-
 #if	defined (_NOORIGGLOB) \
 && !defined (USEREGCMP) && !defined (USEREGCOMP) && !defined (USERE_COMP)
 #undef	_NOORIGGLOB
@@ -107,6 +109,13 @@ typedef struct _hashlist {
 #define	CM_EXE		0020
 #define	CM_ADDEXT	0040
 #define	CM_RECALC	0100
+#define	CM_REHASH	0200
+
+typedef struct _strbuf_t {
+	char *s;
+	ALLOC_T size;
+	ALLOC_T len;
+} strbuf_t;
 
 #ifndef	NODIRLOOP
 typedef struct _devino_t {
@@ -115,30 +124,54 @@ typedef struct _devino_t {
 } devino_t;
 #endif
 
+typedef struct _wild_t {
+	char *s;
+	strbuf_t fixed;
+	strbuf_t path;
+	int quote;
+#ifndef	NODIRLOOP
+	int nino;
+	devino_t *ino;
+#endif
+	u_char flags;
+} wild_t;
+
+#define	EA_STRIPQ	0001
+#define	EA_BACKQ	0002
+#define	EA_KEEPMETA	0004
+#define	EA_NOEVALQ	0010
+#define	EA_STRIPQLATER	0020
+#define	EA_NOUNIQDELIM	0040
+#define	EA_EOLMETA	0100
+
 #ifdef	NOUID_T
-typedef u_short	uid_t;
-typedef u_short	gid_t;
+typedef u_short		uid_t;
+typedef u_short		gid_t;
 #endif
 
 #ifdef	USEPID_T
 typedef pid_t		p_id_t;
-# else
+#else
 typedef long		p_id_t;
 #endif
 
-#if	defined (SIGARGINT) || defined (NOVOID)
-#define	sigarg_t	int
+#ifdef	SIGARGINT
+typedef int		sigarg_t;
 #else
-#define	sigarg_t	void
+# ifdef	NOVOID
+# define		sigarg_t
+# else
+typedef void		sigarg_t;
+# endif
 #endif
 
 #ifdef	SIGFNCINT
-#define	sigfnc_t	int
+typedef int		sigfnc_t;
 #else
 # ifdef	NOVOID
-# define	sigfnc_t
+# define		sigfnc_t
 # else
-# define	sigfnc_t	void
+typedef void		sigfnc_t;
 # endif
 #endif
 
@@ -195,7 +228,9 @@ extern int strnpathcmp2 __P_((char *, char *, int));
 #define	strenvcmp	strcmp
 #define	strnenvcmp	strncmp
 #endif
+extern char *underpath __P_((char *, char *, int));
 extern int isidentchar __P_((int));
+extern int isidentchar2 __P_((int));
 extern int isdotdir __P_((char *));
 extern char *isrootdir __P_((char *));
 extern char *getbasename __P_((char *));
@@ -204,7 +239,7 @@ extern reg_t *regexp_init __P_((char *, int));
 extern int regexp_exec __P_((reg_t *, char *, int));
 extern VOID regexp_free __P_((reg_t *));
 extern int cmppath __P_((CONST VOID_P, CONST VOID_P));
-extern char **evalwild __P_((char *));
+extern char **evalwild __P_((char *, int));
 #ifndef	_NOUSEHASH
 hashlist **duplhash __P_((hashlist **));
 #endif
@@ -232,11 +267,11 @@ extern char *gethomedir __P_((VOID_A));
 #ifndef	MINIMUMSHELL
 extern int evalhome __P_((char **, int, char **));
 #endif
-extern char *evalarg __P_((char *, int, int, int));
+extern char *evalarg __P_((char *, int, int));
 extern int evalifs __P_((int, char ***, char *));
 extern int evalglob __P_((int, char ***, int));
 extern int stripquote __P_((char *, int));
-extern char *_evalpath __P_((char *, char *, int, int));
+extern char *_evalpath __P_((char *, char *, int));
 extern char *evalpath __P_((char *, int));
 
 extern char **argvar;
@@ -259,4 +294,4 @@ extern char *(*posixsubstfunc)__P_((char *, int *));
 extern int pathignorecase;
 #endif
 
-#endif	/* __PATHNAME_H_ */
+#endif	/* !__PATHNAME_H_ */
