@@ -1,9 +1,23 @@
 /*
  *	types.h
  *
- *	Type Definition
+ *	type definitions
  */
 
+#ifndef	__SYS_TYPES_STAT_H_
+#define	__SYS_TYPES_STAT_H_
+#include <sys/types.h>
+#include <sys/stat.h>
+#endif
+
+#ifndef	NOUNISTDH
+#include <unistd.h>
+#endif
+
+#ifndef	NOSTDLIBH
+#include <stdlib.h>
+#endif
+
 #ifdef	USEDIRECT
 #define	dirent	direct
 #endif
@@ -118,6 +132,10 @@ typedef struct _namelist {
 #define	wasmark(file)		((file) -> tmpflags & F_WSMRK)
 #define	isarg(file)		((file) -> tmpflags & F_ISARG)
 #define	havestat(file)		((file) -> tmpflags & F_STAT)
+#define	s_isdir(s)		((((s) -> st_mode) & S_IFMT) == S_IFDIR)
+#define	s_isreg(s)		((((s) -> st_mode) & S_IFMT) == S_IFREG)
+#define	s_islnk(s)		((((s) -> st_mode) & S_IFMT) == S_IFLNK)
+#define	s_isfifo(s)		((((s) -> st_mode) & S_IFMT) == S_IFIFO)
 
 typedef struct _strtable {
 	u_short no;
@@ -242,11 +260,13 @@ typedef struct _winvartable {
 	int v_filepos;
 	int v_sorton;
 	int v_dispmode;
+	int v_fileperrow;
 } winvartable;
 
 extern winvartable winvar[];
 #ifdef	_NOSPLITWIN
 #define	win	0
+#define	windows	1
 #else
 extern int windows;
 extern int win;
@@ -276,22 +296,31 @@ extern int win;
 #define	filepos		(winvar[win].v_filepos)
 #define	sorton		(winvar[win].v_sorton)
 #define	dispmode	(winvar[win].v_dispmode)
+#define	FILEPERROW	(winvar[win].v_fileperrow)
 
 typedef struct _macrostat {
 	short addopt;
 	short needburst;
 	short needmark;
-	u_char flags;
+	u_short flags;
 } macrostat;
 
-#define	F_NOCONFIRM	0001
-#define	F_ARGSET	0002
-#define	F_REMAIN	0004
-#define	F_NOEXT		0010
-#define	F_TOSFN		0020
-#define	F_ISARCH	0040
-#define	F_BURST		0100
-#define	F_MARK		0200
+#define	F_NOCONFIRM	0000001
+#define	F_ARGSET	0000002
+#define	F_REMAIN	0000004
+#define	F_NOEXT		0000010
+#define	F_TOSFN		0000020
+#define	F_ISARCH	0000040
+#define	F_BURST		0000100
+#define	F_MARK		0000200
+#define	F_NOADDOPT	0000400
+#define	F_IGNORELIST	0001000
+#define	F_NOCOMLINE	0002000
+#define	F_NOKANJICONV	0004000
+#define	F_TTYIOMODE	0010000
+#define	F_TTYNL		0020000
+#define	F_EVALMACRO	0040000
+#define	F_DOSYSTEM	0100000
 
 #ifdef	_NOORIGSHELL
 typedef struct _aliastable {
@@ -324,3 +353,11 @@ typedef struct _builtintable {
 #define	isdisplnk(n)		(!archivefile && ((n) & F_SYMLINK))
 #define	isfileflg(n)		(!archivefile && ((n) & F_FILEFLAG))
 #endif
+
+#define	FSID_UFS	1
+#define	FSID_EFS	2
+#define	FSID_SYSV	3
+#define	FSID_FAT	4
+#define	FSID_LFN	5
+#define	FSID_LINUX	6
+#define	FSID_DOSDRIVE	7
