@@ -1,7 +1,7 @@
 /*
  *	unixdisk.c
  *
- *	UNIXlike Disk Access Module
+ *	UNIX-like disk accessing module
  */
 
 #include <stdio.h>
@@ -54,7 +54,6 @@ static char *NEAR strcatdelim2 __P_((char *, char *, char *));
 static int NEAR isdotdir __P_((char *));
 #endif
 
-static int NEAR seterrno __P_((u_int));
 #ifdef	DJGPP
 static int NEAR dos_putpath __P_((char *, int));
 #endif
@@ -105,14 +104,6 @@ static int maxdrive = -1;
 static int dos7access = 0;
 #define	D7_DOSVER	017
 #define	D7_CAPITAL	020
-static u_short doserrlist[] = {
-	0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 15, 18, 65, 80
-};
-#define	DOSERRLISTSIZ	((int)(sizeof(doserrlist) / sizeof(u_short)))
-static int unixerrlist[] = {
-	0, EINVAL, ENOENT, ENOENT, EMFILE, EACCES,
-	EBADF, ENOMEM, ENOMEM, ENOMEM, ENODEV, 0, EACCES, EEXIST
-};
 #if	!defined (DJGPP) && !defined (_NODOSDRIVE)
 static u_char int25bin[] = {
 	0x1e,				/* 0000: push ds */
@@ -148,14 +139,14 @@ static u_char int25bin[] = {
 	0xcb				/* 002a: retf */
 };
 static int (far *doint25)__P_((VOID_A)) = NULL;
-#endif
+#endif	/* !DJGPP && !_NODOSDRIVE */
 
 
 #ifndef	FD
 static int NEAR _dospath(path)
 char *path;
 {
-	return((isalpha(*path) && path[1] == ':') ? *path : 0);
+	return((isalpha2(*path) && path[1] == ':') ? *path : 0);
 }
 
 static char *NEAR strdelim(s, d)
@@ -171,6 +162,7 @@ int d;
 		if (iskanji1(s, i)) i++;
 # endif
 	}
+
 	return(NULL);
 }
 
@@ -189,6 +181,7 @@ int d;
 		if (iskanji1(s, i)) i++;
 # endif
 	}
+
 	return(cp);
 }
 
@@ -207,6 +200,7 @@ int ptr;
 
 	for (i = 0; s[i] && i < ptr; i++) if (iskanji1(s, i)) i++;
 	if (!s[i] || i > ptr) return(1);
+
 	return(!iskanji1(s, i));
 # else
 	return(1);
@@ -241,6 +235,7 @@ char *s;
 		*cp = _SC_;
 	}
 	*(++cp) = '\0';
+
 	return(cp);
 }
 
@@ -291,6 +286,7 @@ char *buf, *s1, *s2;
 		for (i = 0; s2[i] && i < len; i++) *(cp++) = s2[i];
 	}
 	*cp = '\0';
+
 	return(cp);
 }
 
@@ -298,20 +294,11 @@ int isdotdir(s)
 char *s;
 {
 	if (s[0] == '.' && (!s[1] || (s[1] == '.' && !s[2]))) return(1);
+
 	return(0);
 }
 #endif	/* !FD */
 
-static int NEAR seterrno(doserr)
-u_int doserr;
-{
-	int i;
-
-	for (i = 0; i < DOSERRLISTSIZ; i++)
-		if (doserr == doserrlist[i]) return(errno = unixerrlist[i]);
-	return(errno = EINVAL);
-}
-
 #ifdef	DJGPP
 static int NEAR dos_putpath(path, offset)
 char *path;
@@ -321,6 +308,7 @@ int offset;
 
 	i = strlen(path) + 1;
 	dosmemput(path, i, __tb + offset);
+
 	return(i);
 }
 #endif
@@ -350,57 +338,10 @@ char *path;
 		}
 	}
 	buf[j] = '\0';
+
 	return(buf);
 }
 
-int intcall(vect, regp, sregp)
-int vect;
-__dpmi_regs *regp;
-struct SREGS *sregp;
-{
-#ifdef	__TURBOC__
-	struct REGPACK preg;
-#endif
-
-	(*regp).x.flags |= FR_CARRY;
-#ifdef	__TURBOC__
-	preg.r_ax = (*regp).x.ax;
-	preg.r_bx = (*regp).x.bx;
-	preg.r_cx = (*regp).x.cx;
-	preg.r_dx = (*regp).x.dx;
-	preg.r_bp = (*regp).x.bp;
-	preg.r_si = (*regp).x.si;
-	preg.r_di = (*regp).x.di;
-	preg.r_ds = (*sregp).ds;
-	preg.r_es = (*sregp).es;
-	preg.r_flags = (*regp).x.flags;
-	intr(vect, &preg);
-	(*regp).x.ax = preg.r_ax;
-	(*regp).x.bx = preg.r_bx;
-	(*regp).x.cx = preg.r_cx;
-	(*regp).x.dx = preg.r_dx;
-	(*regp).x.bp = preg.r_bp;
-	(*regp).x.si = preg.r_si;
-	(*regp).x.di = preg.r_di;
-	(*sregp).ds = preg.r_ds;
-	(*sregp).es = preg.r_es;
-	(*regp).x.flags = preg.r_flags;
-#else	/* !__TURBOC__ */
-# ifdef	DJGPP
-	(*regp).x.ds = (*sregp).ds;
-	(*regp).x.es = (*sregp).es;
-	__dpmi_int(vect, regp);
-	(*sregp).ds = (*regp).x.ds;
-	(*sregp).es = (*regp).x.es;
-# else
-	int86x(vect, regp, regp, sregp);
-# endif
-#endif	/* !__TURBOC__ */
-	if (!((*regp).x.flags & FR_CARRY)) return(0);
-	seterrno((*regp).x.ax);
-	return(-1);
-}
-
 int getcurdrv(VOID_A)
 {
 	int drive;
@@ -411,6 +352,7 @@ int getcurdrv(VOID_A)
 	else
 #endif
 	drive += (bdos(0x19, 0, 0) & 0xff);
+
 	return(drive);
 }
 
@@ -423,7 +365,7 @@ int drive, nodir;
 	int drv, olddrv;
 
 	errno = EINVAL;
-	if (drive >= 'a' && drive <= 'z') {
+	if (islower2(drive)) {
 		drv = drive - 'a';
 		dos7access &= ~D7_CAPITAL;
 	}
@@ -467,10 +409,10 @@ int drive, nodir;
 			return(-1);
 		}
 	}
-
 #ifndef	_NODOSDRIVE
 	lastdrive = drive + 'a';
 #endif
+
 	return(0);
 }
 
@@ -495,6 +437,7 @@ int getdosver(VOID_A)
 		}
 		dos7access |= (ver & D7_DOSVER);
 	}
+
 	return(dos7access & D7_DOSVER);
 }
 
@@ -521,12 +464,11 @@ char *path;
 #endif
 	if (getdosver() < 7) {
 #ifndef	_NODOSDRIVE
-		if ((dosdrive & 1) && drv[0] >= 'a' && drv[0] <= 'z')
-			return(-1);
+		if ((dosdrive & 1) && islower2(drv[0])) return(-1);
 #endif
 		return(-3);
 	}
-	if (drv[0] >= 'A' && drv[0] <= 'Z') return(0);
+	if (isupper2(drv[0])) return(0);
 
 	drv[1] = ':';
 	drv[2] = '\\';
@@ -553,6 +495,7 @@ char *path;
 	dosmemget(__tb + sizeof(drv), sizeof(buf), buf);
 #endif
 	if (!strcmp(buf, VOL_FAT32)) return(2);
+
 	return(1);
 }
 #endif	/* !_NOUSELFN */
@@ -579,6 +522,7 @@ int drive;
 #ifndef	BSPATHDELIM
 	adjustpname(pathname);
 #endif
+
 	return(pathname);
 }
 
@@ -601,6 +545,7 @@ char *path, *buf;
 	}
 	else if (cp == &(path[2])) return(path);
 	else strcpy(&(buf[2]), cp);
+
 	return(buf);
 }
 # endif	/* !_NODOSDRIVE */
@@ -644,7 +589,7 @@ char *path, *alias;
 # endif
 
 	if (!path || !(i = _dospath(path))) i = getcurdrv();
-	if (i >= 'a' && i <= 'z') *alias = tolower2(*alias);
+	if (islower2(i)) *alias = tolower2(*alias);
 
 	return(alias);
 }
@@ -715,8 +660,9 @@ char *path, *resolved;
 			resolved[0] = i;
 			resolved[1] = ':';
 			i = 2;
-			if (isalpha(resolved[2])
-			&& resolved[3] == '.' && resolved[4] == _SC_) i = 5;
+			if (isalpha2(resolved[2])
+			&& resolved[3] == '.' && resolved[4] == _SC_)
+				i = 5;
 			while (resolved[i]) if (resolved[i++] == _SC_) break;
 			resolved[2] = _SC_;
 			for (j = 0; resolved[i + j]; j++)
@@ -732,7 +678,7 @@ char *path, *resolved;
 				strcpy(&(resolved[2]), path);
 		}
 	}
-	else *resolved = (i >= 'A' && i <= 'Z')
+	else *resolved = (isupper2(i))
 		? toupper2(*resolved) : tolower2(*resolved);
 
 	return(resolved);
@@ -748,6 +694,7 @@ char *path;
 		if (path[i] == '\\') path[i] = _SC_;
 		else if (iskanji1(path, i)) i++;
 	}
+
 	return(path);
 }
 #endif	/* !BSPATHDELIM */
@@ -774,6 +721,7 @@ char *path;
 # endif
 		else path[i] = toupper2(path[i]);
 	}
+
 	return(path);
 }
 #endif	/* DJGPP || !BSPATHDELIM */
@@ -796,6 +744,7 @@ char *path, *alias;
 #ifndef	_NODOSDRIVE
 	else if (i < 0 && errno == EACCES) return(path);
 #endif
+
 	return(NULL);
 }
 
@@ -926,6 +875,7 @@ int nsect, mode, retry;
 	}
 	if (cp != buf) free(cp);
 	errno = EACCES;
+
 	return(-1);
 }
 
@@ -1042,6 +992,7 @@ int nsect, mode, retry;
 	}
 	if (cp != buf) free(cp);
 	errno = EACCES;
+
 	return(-1);
 }
 #endif	/* !PC98 */
@@ -1130,7 +1081,7 @@ u_long l_sect, e_sect;
 		if (PART_TABLE + ofs >= size) {
 #ifndef	PC98
 			if (ps) {
-				if (xbiosdiskio(maxdrive, ps++, 0L,
+				if (xbiosdiskio(maxdrive, ps++, (u_long)0,
 				buf, 1, BIOS_XREAD, BIOSRETRY) < 0
 				&& ((drvlist[maxdrive].flags & DI_INVALIDCHS)
 				|| _biosdiskio(maxdrive, head, sect, cyl,
@@ -1180,14 +1131,15 @@ u_long l_sect, e_sect;
 			continue;
 		}
 		if (pt -> filesys == PT_EXTEND) {
-			if (_checkdrive(sh, ss, sc, 0L, 0L) < 0) {
+			if (_checkdrive(sh, ss, sc, (u_long)0, (u_long)0) < 0)
+			{
 				free(buf);
 				return(-1);
 			}
 			continue;
 		}
 
-		drvlist[maxdrive].f_sect = 0L;
+		drvlist[maxdrive].f_sect = (u_long)0;
 		if (pt -> filesys == PT_FAT16XLBA
 		|| pt -> filesys == PT_FAT32LBA)
 			drvlist[maxdrive].f_sect = ls + l_sect;
@@ -1213,6 +1165,7 @@ u_long l_sect, e_sect;
 		drvlist[maxdrive].flags = drvlist[maxdrive - 1].flags;
 	}
 	free(buf);
+
 	return(1);
 }
 
@@ -1249,14 +1202,18 @@ int drive;
 
 #ifdef	PC98
 		for (i = 0; i < MAX_HDD && maxdrive < 'Z' - 'A' + 1; i++) {
-			if (getdrvparam(BIOS_HDD | i,
-				&(drvlist[maxdrive])) <= 0) continue;
-			if (_checkdrive(0, 1, 0, 0L, 0L) < 0) return(-1);
+			if (getdrvparam(BIOS_HDD | i, &(drvlist[maxdrive]))
+			<= 0)
+				continue;
+			if (_checkdrive(0, 1, 0, (u_long)0, (u_long)0) < 0)
+				return(-1);
 		}
 		for (i = 0; i < MAX_SCSI && maxdrive < 'Z' - 'A' + 1; i++) {
-			if (getdrvparam(BIOS_SCSI | i,
-				&(drvlist[maxdrive])) <= 0) continue;
-			if (_checkdrive(0, 1, 0, 0L, 0L) < 0) return(-1);
+			if (getdrvparam(BIOS_SCSI | i, &(drvlist[maxdrive]))
+			<= 0)
+				continue;
+			if (_checkdrive(0, 1, 0, (u_long)0, (u_long)0) < 0)
+				return(-1);
 		}
 #else
 		reg.h.ah = BIOS_PARAM;
@@ -1265,9 +1222,10 @@ int drive;
 		n = reg.h.dl;
 
 		for (i = 0; i < n && maxdrive < 'Z' - 'A' + 1; i++) {
-			if (!(j = getdrvparam(BIOS_HDD | i,
-				&(drvlist[maxdrive])))) continue;
-			if (j < 0 || _checkdrive(0, 0, 0, 0L, 0L) < 0)
+			j = getdrvparam(BIOS_HDD | i, &(drvlist[maxdrive]));
+			if (!j) continue;
+			if (j < 0
+			|| _checkdrive(0, 0, 0, (u_long)0, (u_long)0) < 0)
 				return(-1);
 		}
 #endif
@@ -1275,6 +1233,7 @@ int drive;
 	if (drive >= 0 && drive < maxdrive
 	&& (drvlist[drive].flags & DI_PSEUDO))
 		return(drvlist[drive].filesys);
+
 	return(0);
 }
 
@@ -1320,10 +1279,12 @@ int n, size, iswrite;
 	head = sect / drvlist[drive].sect;
 	sect %= drvlist[drive].sect;
 
-	if (!iswrite) return(_biosdiskio(drive, head, sect, cyl,
-		buf, n, BIOS_READ, BIOSRETRY));
+	if (!iswrite)
+		return(_biosdiskio(drive, head, sect, cyl,
+			buf, n, BIOS_READ, BIOSRETRY));
 	else if (_biosdiskio(drive, head, sect, cyl,
-	buf, n, BIOS_WRITE, BIOSRETRY) < 0) return(-1);
+	buf, n, BIOS_WRITE, BIOSRETRY) < 0)
+		return(-1);
 	else return(_biosdiskio(drive, head, sect, cyl,
 		buf, n, BIOS_VERIFY, 1));
 }
@@ -1362,6 +1323,7 @@ int drv;
 			return(-1);
 		}
 	}
+
 	return(end);
 }
 
@@ -1383,6 +1345,7 @@ int drv, level;
 		reg.x.cx = 0x086a;
 		if (int21call(&reg, &sreg) < 0) return(-1);
 	}
+
 	return(0);
 }
 
@@ -1505,6 +1468,7 @@ int n, size, iswrite;
 #ifdef	DJGPP
 	if (!iswrite) dosmemget(__tb + sizeof(struct iopacket_t), size, buf);
 #endif
+
 	return(0);
 }
 
@@ -1556,6 +1520,7 @@ int n, size, iswrite;
 		ret = dosrawdiskio(drv, sect, buf, n, size, iswrite);
 		if (iswrite && unlockdrive(drv, level) < 0) return(-1);
 	}
+
 	return(ret);
 }
 #endif	/* !_NODOSDRIVE */
@@ -1586,6 +1551,7 @@ struct dosfind_t *result;
 #ifdef	DJGPP
 	dosmemget(__tb, sizeof(struct dosfind_t), result);
 #endif
+
 	return(0);
 }
 
@@ -1608,6 +1574,7 @@ struct dosfind_t *result;
 #ifdef	DJGPP
 	dosmemget(__tb, sizeof(struct dosfind_t), result);
 #endif
+
 	return(0);
 }
 
@@ -1637,6 +1604,7 @@ struct lfnfind_t *result;
 #ifdef	DJGPP
 	dosmemget(__tb, sizeof(struct lfnfind_t), result);
 #endif
+
 	return(0);
 }
 
@@ -1656,6 +1624,7 @@ struct lfnfind_t *result;
 #ifdef	DJGPP
 	dosmemget(__tb, sizeof(struct lfnfind_t), result);
 #endif
+
 	return(0);
 }
 
@@ -1667,6 +1636,7 @@ u_int fd;
 
 	reg.x.ax = 0x71a1;
 	reg.x.bx = fd;
+
 	return(int21call(&reg, &sreg));
 }
 #endif	/* !_NOUSELFN */
@@ -1753,6 +1723,7 @@ char *dir;
 		return(NULL);
 	}
 	strcpy(dirp -> dd_path, dir);
+
 	return(dirp);
 }
 
@@ -1768,6 +1739,7 @@ DIR *dirp;
 	free(dirp -> dd_buf);
 	free(dirp -> dd_path);
 	free(dirp);
+
 	return(0);
 }
 
@@ -1881,6 +1853,7 @@ int mode;
 		errno = EEXIST;
 		return(-1);
 	}
+
 	return(mkdir(path) ? -1 : 0);
 }
 # endif
@@ -1912,6 +1885,7 @@ char *path;
 #endif
 	sreg.ds = PTR_SEG(path);
 	reg.x.dx = PTR_OFF(path, 0);
+
 	return(int21call(&reg, &sreg));
 }
 
@@ -1942,6 +1916,7 @@ char *s;
 	}
 	if (i) *(s++) = '.';
 	s[i] = '\0';
+
 	return(1);
 }
 
@@ -1964,7 +1939,8 @@ char *from, *to;
 		tp = &(to[tp - fbuf]);
 	}
 	if (!(to = unixrealpath(fbuf, tbuf))
-	|| !(from = unixrealpath(from, fbuf))) return(-1);
+	|| !(from = unixrealpath(from, fbuf)))
+		return(-1);
 	strcpy(strcatdelim(tbuf), tp);
 
 	for (i = 0; from[i]; i++) if (from[i] != to[i]) break;
@@ -2039,7 +2015,8 @@ char *from, *to;
 		if (i == 1) {
 			if (ax != 0x5600
 			|| dos_findfirst(from, SEARCHATTRS, &dbuf) < 0
-			|| !(dbuf.attr & DS_IFDIR)) break;
+			|| !(dbuf.attr & DS_IFDIR))
+				break;
 
 			if (t > -3) {
 				if ((f = gendosname(to)) < 0) break;
@@ -2053,6 +2030,7 @@ char *from, *to;
 		}
 	}
 	errno = EACCES;
+
 	return(-1);
 }
 
@@ -2084,6 +2062,7 @@ int mode;
 	reg.x.dx = PTR_OFF(path, 0);
 	if (int21call(&reg, &sreg) >= 0) return(0);
 	if (reg.x.ax == 5) errno = EEXIST;
+
 	return(-1);
 }
 
@@ -2109,6 +2088,7 @@ char *path;
 #endif
 	sreg.ds = PTR_SEG(path);
 	reg.x.dx = PTR_OFF(path, 0);
+
 	return(int21call(&reg, &sreg));
 }
 
@@ -2135,6 +2115,7 @@ char *path;
 #endif
 	sreg.ds = PTR_SEG(path);
 	reg.x.dx = PTR_OFF(path, 0);
+
 	return(int21call(&reg, &sreg));
 }
 #endif	/* !_NOUSELFN */
@@ -2169,6 +2150,7 @@ int size;
 #ifdef	DJGPP
 	if (unixrealpath(pathname, tmp)) strcpy(pathname, tmp);
 #endif
+
 	return(pathname);
 }
 
@@ -2393,6 +2375,7 @@ struct stat *stp;
 	stp -> st_dev = stp -> st_ino = 0;
 	stp -> st_nlink = 1;
 	stp -> st_uid = stp -> st_gid = -1;
+
 	return(0);
 }
 
@@ -2430,6 +2413,7 @@ int mode;
 #endif
 	sreg.ds = PTR_SEG(path);
 	reg.x.dx = PTR_OFF(path, 0);
+
 	return(int21call(&reg, &sreg));
 }
 
@@ -2547,6 +2531,7 @@ int flags, mode;
 	int21call(&reg, &sreg);
 
 	if (!(path = shortname(path, buf))) return(-1);
+
 	return(open(path, flags, mode));
 }
 
@@ -2594,6 +2579,7 @@ char *path, *type;
 	}
 
 	if ((i = unixopen(path, flags, 0666)) < 0) return(NULL);
+
 	return(fdopen(i, type));
 }
 #endif	/* !_NOUSELFN */
